package com.huojutech.controller.weixin;

import com.github.pagehelper.PageHelper;
import com.huojutech.constant.SessionConstants;
import com.huojutech.dto.AddCollectionDTO;
import com.huojutech.dto.CollectionDTO;
import com.huojutech.entity.StockForecast;
import com.huojutech.entity.StockForecastCollect;
import com.huojutech.entity.User;
import com.huojutech.service.CollectionService;
import com.huojutech.service.ForecastService;
import com.huojutech.service.UserService;
import com.huojutech.utils.DateUtil;
import com.huojutech.utils.ResultEnum;
import com.huojutech.vo.PageResultVO;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.text.DecimalFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

/**
 * Created by Wangzw on 2018\4\17 0017.
 */
@RestController
@Api(description = "我的收藏")
@RequestMapping("/wx/collection")
public class CollectionController {

    @Autowired
    private CollectionService myCollectionService;
    
    @Autowired
    private UserService userService;

    @Autowired
    private ForecastService forecastService;

    @SuppressWarnings({ "rawtypes", "unchecked" })
    @RequestMapping(value = "/list", produces = "application/json;charset=utf-8", method = RequestMethod.GET)
    @ApiOperation(value = "查询用户收藏", notes = "查询收藏根据uid", response = CollectionDTO.class)
    public PageResultVO<CollectionDTO> myCollection(
            @ApiParam(value = "分页大小,默认10", required = false) @RequestParam(required = false, defaultValue = "10") Integer size,
            @ApiParam(value = "页码 ，第一页为1") @RequestParam(required = false, defaultValue = "1") Integer page) throws ParseException {
        Integer sessionId = SessionConstants.getSession();
        List<CollectionDTO> listDTO = new ArrayList<>();
        PageResultVO vo = new PageResultVO();
        PageHelper.startPage(page,size);
        List<StockForecastCollect> collectionList = myCollectionService.findCollectionList(sessionId);
        if (null!=collectionList && collectionList.size()>0){
            for (StockForecastCollect result:collectionList) {
                CollectionDTO myCollectionDTO = new CollectionDTO();
                Integer forecastId = result.getForecastId();//预测id
                StockForecast stockForecast= forecastService.findForecastById(forecastId);
                Integer uid1 = stockForecast.getUid();//用户id
                if (uid1.equals(sessionId)){
                    continue;
                }
                User user = userService.selectUserById(uid1);

                Double forecastRateSuccess = user.getForecastRateSuccess();
                //小数转百分数
                DecimalFormat df = new DecimalFormat("0.00%");
                String forecastSuc = df.format(forecastRateSuccess);

                String nickname = user.getNickname();//昵称
                String avatar = user.getAvatar();//头像

                Date startDate = stockForecast.getStartDate();
                Date endDate = stockForecast.getEndDate();

                SimpleDateFormat format = new SimpleDateFormat("yyyy.MM.dd");
                String startStringDate = format.format(startDate);
                String endStringDate = format.format(endDate);

                String timeZone = startStringDate+"-"+endStringDate;//时间区间拼接

                Date todayTime = new Date();
                String format1 = format.format(todayTime);
                Date parse =null;
                Date parse2=null;

                parse = format.parse(format1);//当前时间
                parse2 = format.parse(endStringDate);//预测结束时间

                int restTime = DateUtil.diffDate(parse, parse2);//距离到期时间

                myCollectionDTO.setName(nickname);
                myCollectionDTO.setExpect(stockForecast.getExpect());
                myCollectionDTO.setCrtTime(result.getCrtTime());
                myCollectionDTO.setCycle(stockForecast.getCycle());
                myCollectionDTO.setTimeZone(timeZone);
                myCollectionDTO.setReason(stockForecast.getReason());
                myCollectionDTO.setStatus(stockForecast.getStatus());
                myCollectionDTO.setTargetPrice(stockForecast.getTargetPrice());
                myCollectionDTO.setRestTime(restTime);
                myCollectionDTO.setHistorySuc(forecastSuc);
                myCollectionDTO.setAvatar(avatar);
                myCollectionDTO.setId(forecastId);
                listDTO.add(myCollectionDTO);
            }
        }
        vo.setData(listDTO);
        vo.setCode(ResultEnum.SUCCESS.getCode());
        vo.setMsg(ResultEnum.SUCCESS.getMessage());
        return vo;
    }

    @SuppressWarnings("rawtypes")
    @RequestMapping(value = "/add", produces = "application/json;charset=utf-8", method = RequestMethod.POST)
    @ApiOperation(value = "用户添加收藏", notes = "添加收藏")
    public PageResultVO Collection(@RequestBody AddCollectionDTO collectionDTO) {
        Integer sessionId = SessionConstants.getSession();
        PageResultVO vo = new PageResultVO();
        if (collectionDTO.getFlag()) {
            //flag==true,已关注，取消操作
            List<StockForecastCollect> collects = myCollectionService.findCollectionList(sessionId);
            Integer result = 0;
            for (StockForecastCollect collection : collects) {
                Integer id = collection.getForecastId();//预测id
                Integer id1 = collection.getId();//收藏id
                if (id.equals(collectionDTO.getForecastId())) {
                    result = myCollectionService.deleteByUid(id1);
                }
            }
            if (result.equals(1)) {
                vo.setCode(ResultEnum.SUCCESS.getCode());
                vo.setMsg(ResultEnum.SUCCESS.getMessage());
                return vo;
            }

        } else {
            //flag==false,未关注
            StockForecastCollect collect = new StockForecastCollect();
            collect.setUid(sessionId);
            collect.setForecastId(collectionDTO.getForecastId());
            Integer suc = myCollectionService.insertConcern(collect);

            if (suc.equals(1)) {
                vo.setCode(ResultEnum.SUCCESS.getCode());
                vo.setMsg(ResultEnum.SUCCESS.getMessage());
                return vo;
            }
        }
        return vo;
    }
}


