package com.huojutech.controller.weixin;

import com.github.pagehelper.PageHelper;
import com.github.pagehelper.StringUtil;
import com.huojutech.constant.Constants;
import com.huojutech.constant.SessionConstants;
import com.huojutech.dto.*;
import com.huojutech.entity.StockForecast;
import com.huojutech.entity.SystemConfig;
import com.huojutech.entity.User;
import com.huojutech.entity.UserConcern;
import com.huojutech.service.*;
import com.huojutech.utils.DateUtil;
import com.huojutech.utils.ResultEnum;
import com.huojutech.vo.PageResultVO;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.util.CollectionUtils;
import org.springframework.web.bind.annotation.*;

import javax.script.ScriptEngine;
import javax.script.ScriptEngineManager;
import javax.script.ScriptException;
import java.math.BigDecimal;
import java.text.DecimalFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

/**
 * Created by Wangzw on 2018\4\19 0019.
 */
@RestController
@Api(description = "我的预测接口")
@RequestMapping("/wx/forecast")
@SuppressWarnings({ "rawtypes", "unchecked","unused" })
public class ForecastController {

    @Autowired
    private ForecastService forecastService;

    @Autowired
    private UserService userService;

    @Autowired
    private UserConcernService userConcernService;

    @Autowired
    private StockOptionalService stockOptionalService;

    @Autowired
    private StockService stockService;

    @Autowired
    private SystemConfigService systemConfigService;

    private static ScriptEngine jse = new ScriptEngineManager().getEngineByName("JavaScript");

    @RequestMapping(value = "/list", produces = "application/json;charset=utf-8", method = RequestMethod.GET)
    @ApiOperation(value = "查询用户预测股票的数量", notes = "根据传来的用户id，和预测状态status，0:预测中，1:已经结束", response = MyForecastDTO.class)
    public PageResultVO<MyForecastDTO> forecast(
            @ApiParam(value = "预测状态0:预测中，1:已经结束") @RequestParam Integer status,
            @ApiParam(value = "分页大小,默认10", required = false) @RequestParam(required = false, defaultValue = "10") Integer size,
            @ApiParam(value = "页码 ，第一页为1") @RequestParam(required = false, defaultValue = "1") Integer page) throws ParseException, ScriptException {
        Integer uid = SessionConstants.getSession();
        List<MyForecastDTO> list = new ArrayList<>();
        PageHelper.startPage(page,size);
        List<StockForecast> forecastInfo = null;
        if (status.equals(0)) {
            forecastInfo = forecastService.findForecastOn(uid,status);
        }else {
            List<Integer> numList = new ArrayList<>();
            numList.add(1);
            numList.add(2);
            forecastInfo = forecastService.findForecastOff(uid,numList);
        }

        SimpleDateFormat format = new SimpleDateFormat("yyyy.MM.dd");

        for (StockForecast infoList:forecastInfo) {
            MyForecastDTO forecastDto = new MyForecastDTO();

            Date startDate = infoList.getStartDate();//开始时间
            Date endDate = infoList.getEndDate();//结束时间
            String startStringDate = format.format(startDate);
            String endStringDate = format.format(endDate);
            String timeZone = startStringDate+ "-" +endStringDate;

            Date todayTime = new Date();
            String format1 = format.format(todayTime);
            Date parse =null;
            Date parse2=null;

            parse = format.parse(format1);//当前时间
            parse2 = format.parse(endStringDate);//预测结束时间
            int restTime = DateUtil.diffDate(parse, parse2);

            String stockName = infoList.getStockName();
            Integer vnum = infoList.getVnum();//此条预测的被投票数
            Integer id1 = infoList.getId();//此次预测的id

            Integer status1 = infoList.getStatus();//状态

            if (!status.equals(0)){
                forecastDto.setHarvest(infoList.getHarvest());
            }else {
                //预测进行中，返回如果成功规则计算出的应得火钻
                BigDecimal bet = infoList.getBet();
                BigDecimal currentPrice = infoList.getCurrentPrice();
                BigDecimal targetPrice = infoList.getTargetPrice();
                BigDecimal subtract = targetPrice.subtract(currentPrice);
                BigDecimal divide = new BigDecimal(0);
                //数据库中有股价为0的情况
                int i1 = currentPrice.compareTo(BigDecimal.ZERO);
                if (i1!=0){
                    int i = subtract.compareTo(BigDecimal.ZERO);
                    if (i==-1){
                        String substring = subtract.toString().substring(1);
                        subtract = new BigDecimal(substring);
                    }
                    divide = subtract.divide(currentPrice,2,BigDecimal.ROUND_HALF_UP);
                }else {
                    divide = subtract;
                }


                //#bet#*(1+#changepercent#)+#vbet#*0.02
                List<SystemConfig> systemConfigs = systemConfigService.selectByKey(Constants.CONFIG_FORECAST_FORMULA);
                if(!CollectionUtils.isEmpty(systemConfigs) && StringUtil.isNotEmpty(systemConfigs.get(0).getCfgVal())){
                    SystemConfig systemConfig = systemConfigs.get(0);
                    String cfgVal = systemConfig.getCfgVal();
                    cfgVal = cfgVal.replace("#bet#", bet+"").replace("#changepercent#", divide+"").replace("#vbet#",vnum+"");
                    String have = String.format("%.6f", jse.eval(cfgVal));
                    BigDecimal harvest = new BigDecimal(have);
                    harvest = harvest.setScale(6,BigDecimal.ROUND_HALF_UP);

                    forecastDto.setHarvest(harvest);
                }
            }
            BigDecimal voteHarvest = infoList.getVoteHarvest();
            if (voteHarvest!=null && !voteHarvest.equals(0)  ){
                BigDecimal bigDecimal = voteHarvest.setScale(6, BigDecimal.ROUND_HALF_UP);
                forecastDto.setVoteHarvest(bigDecimal);
            }else {
                forecastDto.setVoteHarvest(voteHarvest);
            }

            forecastDto.setId(id1);
            forecastDto.setStockName(stockName);//股票名字
            forecastDto.setExpect(infoList.getExpect());//看多看空
            forecastDto.setCycle(infoList.getCycle());//预测周期
            forecastDto.setTargetPrice(infoList.getTargetPrice());//目标价格
            forecastDto.setTimeZone(timeZone);//时间区间
            forecastDto.setVoteNum(vnum);//获得的投票数

            forecastDto.setRestTime(restTime);
            forecastDto.setMyStatus(status1);
            forecastDto.setBet(infoList.getBet());//本金

            list.add(forecastDto);
        }
        PageResultVO vo = new PageResultVO();
        vo.setData(list);
        vo.setCode(ResultEnum.SUCCESS.getCode());
        vo.setMsg(ResultEnum.SUCCESS.getMessage());
        return vo;
    }

    @RequestMapping(value = "/seerList", produces = "application/json;charset=utf-8", method = RequestMethod.GET)
    @ApiOperation(value = "查询预言家列表", notes = "预言家列表", response = SeerListDTO.class)
    public PageResultVO<SeerListDTO> seerList(
            @ApiParam(value = "分页大小,默认10", required = false) @RequestParam(required = false, defaultValue = "10") Integer size,
            @ApiParam(value = "页码 ，第一页为1") @RequestParam(required = false, defaultValue = "1") Integer page,
            @ApiParam(value = "股票名称或股票代码(模糊查询)", required = false) @RequestParam(required = false) String nickName,
            @ApiParam(value = "股票代码（查询此股票下的预测人列表时用到）", required = false) @RequestParam(required = false) String stockCode,
            @ApiParam(value = "预测数排序规则，0-升序  1-降序 (注：无法与priceOrderby、cpOrderby合用)", required = false) @RequestParam(required = false) Integer fnumOrderby,
            @ApiParam(value = "预测成功率排序规则，0-升序  1-降序 (注：无法与frateOrderby、concernOrderby合用))", required = false) @RequestParam(required = false) Integer frateOrderby,
            @ApiParam(value = "被关注数排序规则，0-升序  1-降序 (注：无法与fnumOrderby、fnumOrderby合用))", required = false) @RequestParam(required = false) Integer concernOrderby,
            @ApiParam(value = "是否是查询订阅的预言家，1：查询订阅的预言家 ，null：不是", required = false) @RequestParam(required = false) Integer isSubscribe,
            @ApiParam(value = "更多预言家推荐，1：查询更多预言家推荐 ，null:不查，无法与isSubscribe合用" , required = false) @RequestParam(required = false) Integer recommend) throws ParseException {
        //获取所有的预测人的集合
        Integer sessionId = SessionConstants.getSession();
        List<StockForecast> allForecast = forecastService.findAllForecastUID();
        List<UserConcern> concerns = userConcernService.selectAllConcernCUID(sessionId);
        List<User> recommondUid = userService.selectRecommondUid();
        List<SeerListDTO> list = new ArrayList<>();
        PageHelper.startPage(page, size);
        if (isSubscribe!=null && isSubscribe.equals(1)){
            list = forecastService.findAllConcern(concerns, nickName, fnumOrderby, frateOrderby, concernOrderby,stockCode);
        }else if (recommend!=null && recommend.equals(1)){
            list = forecastService.findAllRecommend(recommondUid,nickName,fnumOrderby,frateOrderby,concernOrderby,stockCode);
        }else {
            list = forecastService.findAllForecast(allForecast, nickName, fnumOrderby, frateOrderby, concernOrderby,stockCode);
        }
        for (SeerListDTO listDTO : list) {
            String forecast = listDTO.getForecastSuc();
            double v = Double.parseDouble(forecast);
            DecimalFormat df = new DecimalFormat("0.00%");
            String forecastSuc = df.format(v);
            listDTO.setForecastSuc(forecastSuc);
        }

        PageResultVO vo = new PageResultVO();
        vo.setData(list);
        vo.setCode(ResultEnum.SUCCESS.getCode());
        vo.setMsg(ResultEnum.SUCCESS.getMessage());
        return vo;
    }


    @RequestMapping(value = "/seerList/detail/{id}", produces = "application/json;charset=utf-8", method = RequestMethod.GET)
    @ApiOperation(value = "查询预言家详情", notes = "预言家详情", response = SeerDetailDTO.class)
    public PageResultVO<SeerDetailDTO> seerDetail(
            @ApiParam(value = "分页大小,默认10", required = false) @RequestParam(required = false, defaultValue = "10") Integer size,
            @ApiParam(value = "页码 ，第一页为1") @RequestParam(required = false, defaultValue = "1") Integer page,
            @ApiParam(value = "用户id") @PathVariable(value = "id") Integer id) throws ParseException {
        Integer sessionId = SessionConstants.getSession();
        SeerDetailDTO seerDetailDTO = new SeerDetailDTO();
        List<SeerDetailListDTO> forecastList = new ArrayList();
        Integer optionalCounts = stockOptionalService.selectOptionalCounts(id);
        User user = userService.selectUserById(id);
        String nickname = user.getNickname();//昵称
        String avatar = user.getAvatar();//头像
        Integer fnum = user.getFnum();//预测数量
        Integer vnum = user.getVnum();//获得投票的数量
        Integer concernNum = user.getConcernNum();//关注别人的数量
        Integer cnum = user.getCnum();//粉丝数

        Double forecastRateSuccess = user.getForecastRateSuccess();
        Double voteRateSuccess = user.getVoteRateSuccess();

        DecimalFormat df = new DecimalFormat("0.00%");
        String forecastSuc = df.format(forecastRateSuccess);//预测成功率
        String voteRateSuc = df.format(voteRateSuccess);//投票成功率

        List<UserConcern> concerns = userConcernService.selectFansList(id);
        Boolean flag = false;//是否关注，默认否,没有关注
        if (!concerns.isEmpty()) {
            for (UserConcern userConcern : concerns) {
                Integer fanId = userConcern.getUid();
                if (fanId.equals(sessionId)) {
                    flag = true;
                    break;
                }
            }
        }
        SimpleDateFormat format = new SimpleDateFormat("yyyy.MM.dd");
        PageHelper.startPage(page,size);
        List<StockForecast> seerForecast = forecastService.findSeerForecast(id);
        for (StockForecast forecast : seerForecast) {
            SeerDetailListDTO seerDetailListDTO = new SeerDetailListDTO();
            Integer id1 = forecast.getId();//预测的这支股票的id
            Integer expect = forecast.getExpect();//看多看空
            BigDecimal targetPrice = forecast.getTargetPrice();//目标价格
            Integer cycle = forecast.getCycle();//关注周期，一周，两周
            String reason = forecast.getReason();//预测理由
            Date startDate = forecast.getStartDate();//预测开始时间
            Date endDate = forecast.getEndDate();//预测结束时间

            String startStringDate = format.format(startDate);
            String endStringDate = format.format(endDate);
            String timeZone = startStringDate+"-"+endStringDate;//预测周期时间区间

            Date todayTime = new Date();
            String format1 = format.format(todayTime);
            Date parse = format.parse(format1);//当前时间
            int restTime = DateUtil.diffDate(parse, endDate);//剩下多少时间
            String weiboTime = DateUtil.getWeiboTime(forecast.getCrtTime());//更新时间

            seerDetailListDTO.setId(id1);
            seerDetailListDTO.setCycle(cycle);
            seerDetailListDTO.setExpect(expect);
            seerDetailListDTO.setForecastSuc(forecastSuc);
            seerDetailListDTO.setRestTime(restTime);
            seerDetailListDTO.setTargetPrice(targetPrice);
            seerDetailListDTO.setTimeZone(timeZone);
            seerDetailListDTO.setCrtTime(weiboTime);
            seerDetailListDTO.setReason(reason);
            seerDetailListDTO.setStockName(forecast.getStockName());
            seerDetailListDTO.setStockCode(forecast.getStockCode());
            seerDetailListDTO.setStatus(forecast.getStatus());
            forecastList.add(seerDetailListDTO);
        }
        seerDetailDTO.setId(id);//点击的预言家id
        seerDetailDTO.setUid(sessionId);//小程序用户id
        seerDetailDTO.setNickName(nickname);
        seerDetailDTO.setAvatar(avatar);
        seerDetailDTO.setUserConcernNum(concernNum);
        seerDetailDTO.setFansNum(cnum);
        seerDetailDTO.setConcern(flag);
        seerDetailDTO.setForecastNum(fnum);
        seerDetailDTO.setVoteNum(vnum);
        seerDetailDTO.setOptionalNum(optionalCounts);
        seerDetailDTO.setForecastSuc(forecastSuc);
        seerDetailDTO.setVoteSuc(voteRateSuc);
        seerDetailDTO.setForeacstList(forecastList);

        PageResultVO vo = new PageResultVO();
        vo.setData(seerDetailDTO);
        vo.setCode(ResultEnum.SUCCESS.getCode());
        vo.setMsg(ResultEnum.SUCCESS.getMessage());
        return vo;
    }

    @RequestMapping(value = "/seerList/concern", produces = "application/json;charset=utf-8", method = RequestMethod.POST)
    @ApiOperation(value = "预言家关注", notes = "点击关注、取消关注预言家")
    public PageResultVO concern(@RequestBody ConcernDTO concernDTO) {
        Integer sessionId = SessionConstants.getSession();
        PageResultVO vo  = new PageResultVO();
        if (concernDTO.getFlag()){
            //flag==true，点击即是取消操作
            userConcernService.deleteConcern(sessionId,concernDTO.getId());
        }else {
            //flag==false，点击是关注操作
            userConcernService.insertConcern(sessionId,concernDTO.getId());
        }
        vo.setCode(ResultEnum.SUCCESS.getCode());
        vo.setMsg(ResultEnum.SUCCESS.getMessage());
        return vo;
    }
}
