package com.huojutech.controller.weixin;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.huojutech.annotation.SkipLogin;
import com.huojutech.constant.Constants;
import com.huojutech.constant.SessionConstants;
import com.huojutech.constant.wx.UserInfo;
import com.huojutech.dto.LoginDTO;
import com.huojutech.entity.DiamondRecord;
import com.huojutech.entity.User;
import com.huojutech.request.RequestSessionKey;
import com.huojutech.service.DiamondRecordService;
import com.huojutech.service.SystemConfigService;
import com.huojutech.service.UserService;
import com.huojutech.utils.HttpsUtil;
import com.huojutech.utils.ResultEnum;
import com.huojutech.utils.ResultVOUtil;
import com.huojutech.vo.LoginVO;
import com.huojutech.vo.ResultVO;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.data.redis.core.ValueOperations;
import org.springframework.web.bind.annotation.*;

import java.math.BigDecimal;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.List;

/**
 * Created by Wangzw on 2018\4\16 0016.
 */

@RestController
@Api(description = "用户登录接口")
public class LoginController {
    private static Logger logger = LoggerFactory.getLogger(LoginController.class);

    
    @Autowired
    private UserService userService;

    @Autowired
    private DiamondRecordService diamondRecordService;

    @Autowired
    private SystemConfigService systemConfigService;

    @Autowired
    private RedisTemplate redisTemplate;

    @SuppressWarnings({ "unchecked" })
    @RequestMapping(value ="/wx/onLogin",produces = "application/json;charset=utf-8",method = RequestMethod.POST)
    @ApiOperation(value="用户登录",notes="根据传来的code以及用户身份信息登录",response = LoginVO.class)
    @SkipLogin
    public ResultVO<LoginVO> login(@RequestBody LoginDTO loginDTO) {
        LoginVO loginVO = new LoginVO();
        //获取用户openid的实际网址
        String token = UserInfo.getWebAccess(loginDTO.getCode());
        //通过HttpGet方法将token发送至微信服务器并获得其回执
        String rec = HttpsUtil.httpsRequestToString(token, "GET", null);
        //使用json类解析结果
        JSONObject json = JSON.parseObject(rec);
        //获取回执的openid
        if (null != json && null != json.getString("openid")) {
            //获取openid
            String openid = json.getString("openid");
            String unionid = json.getString("unionid");
            String sessionKey = json.getString("session_key");
            ValueOperations valueOperations = redisTemplate.opsForValue();
            valueOperations.set("sessionKey",sessionKey);
            //TODO 用户登录操作类型
            List<User> userInfoList = userService.findUserByOpenId(openid);
            User userInfo=null;
            if (!userInfoList.isEmpty()){
                userInfo = userInfoList.get(0);
            }
            User user = new User();
            DiamondRecord diamondRecord = new DiamondRecord();
            SimpleDateFormat format = new SimpleDateFormat("yyyy-MM-dd");
            if (userInfo != null) {
                //每日登录，获取最后登录时间，判断今日有没有登录
                Date lastSignTime = userInfo.getLastSignTime();
                Date today = new Date();
                String lastSign = format.format(lastSignTime);
                String todayTime = format.format(today);
                //不相等，表今天没有登录
                if (!lastSign.equals(todayTime)){
                    BigDecimal amount1 = userInfo.getAmount();
                    BigDecimal add = amount1.add(Constants.SIGN_TODAY_Y);

                    //用户表加火钻
                    userInfo.setAmount(add);
                    userInfo.setLastSignTime(today);
                    userService.updateUser(userInfo);
                    //火钻记录表插入数据
                    diamondRecord.setUid(userInfo.getId());
                    diamondRecord.setName("每日登录");
                    diamondRecord.setDescr("每日登录送火钻！");
                    diamondRecord.setAmount(Constants.SIGN_TODAY_Y);
                    diamondRecordService.insertSelective(diamondRecord);
                }else {
                    //相等表已经登录，更新最后登录时间
                    userInfo.setLastSignTime(today);
                    userService.updateUser(userInfo);
                }
                Integer id = userInfo.getId();
                loginVO.setUserId(id);
                loginVO.setOpenId(openid);
                loginVO.setToken(RequestSessionKey.getSessionKey());
                SessionConstants.setSession(userInfo);
            } else {
                //用户信息为空表示首次登陆
                BigDecimal signFirstH = Constants.SIGN_FIRST_H;
                BigDecimal signTodayY = Constants.SIGN_TODAY_Y;
                BigDecimal add = signFirstH.add(signTodayY);
                //插入用户数据
                user.setWxOpenid(openid);
                user.setStatus(0);
                user.setAmount(add);
                Integer num = userService.saveUser(user);
                //插入火钻记录表数据
                diamondRecord.setUid(user.getId());
                diamondRecord.setName("注册");
                diamondRecord.setDescr("用户首次注册");
                diamondRecord.setAmount(signFirstH);
                Integer result = diamondRecordService.insertSelective(diamondRecord);
                diamondRecord.setName("登录");
                diamondRecord.setDescr("每日登录送火钻!");
                diamondRecord.setAmount(signTodayY);
                diamondRecordService.insertSelective(diamondRecord);
                //用户邀请表插入数据
                if (!num.equals(1)){
                    return ResultVOUtil.error(ResultEnum.INFO_SAVE_FAILED.getCode(),ResultEnum.INFO_SAVE_FAILED.getMessage());
                }
                if (!result.equals(1)){
                    return ResultVOUtil.error(ResultEnum.INFO_SAVE_FAILED.getCode(),ResultEnum.INFO_SAVE_FAILED.getMessage());
                }

                Integer id = user.getId();
                loginVO.setUserId(id);
                loginVO.setOpenId(openid);
                loginVO.setToken(RequestSessionKey.getSessionKey());
                SessionConstants.setSession(user);
            }
        }
        return ResultVOUtil.success(loginVO);
    }


    @SuppressWarnings({ "unchecked" })
    @RequestMapping(value ="/wx/onLogin_bbb",produces = "application/json;charset=utf-8",method = RequestMethod.GET)
    @SkipLogin
    public ResultVO<LoginVO> backlogin(@RequestParam  String openId) {
        LoginVO loginVO = new LoginVO();
        
        User user = userService.getUserByOpenId(openId);
        if (user == null) {
            return ResultVOUtil.error(ResultEnum.INFO_SAVE_FAILED.getCode(),ResultEnum.INFO_SAVE_FAILED.getMessage());
        }
        
        SessionConstants.setSession(user);
        loginVO.setOpenId(openId);
        loginVO.setToken(RequestSessionKey.getSessionKey());
        return ResultVOUtil.success(loginVO);
    }
}

