package com.huojutech.controller.weixin;

import com.huojutech.constant.Constants;
import com.huojutech.constant.SessionConstants;
import com.huojutech.dto.HotStockDTO;
import com.huojutech.entity.Stock;
import com.huojutech.entity.StockOptional;
import com.huojutech.entity.StockSimple;
import com.huojutech.service.StockOptionalService;
import com.huojutech.service.StockService;
import com.huojutech.utils.DateUtil;
import com.huojutech.utils.ResultEnum;
import com.huojutech.vo.PageResultVO;
import com.huojutech.vo.ResultVO;
import com.huojutech.vo.StockKVO;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.apache.commons.lang3.time.DateUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.util.CollectionUtils;
import org.springframework.web.bind.annotation.*;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

/**
 * 股票相关接口
 * 
 * @author zhoucong
 */
@RestController
@Api(description = "股票相关接口")
@RequestMapping(value = "/wx/stock/*")
public class StockController {
    private static Logger logger = LoggerFactory.getLogger(StockController.class);
    @Autowired
    private StockService stockService;
    @Autowired
    private StockOptionalService stockOptionalService;
    
    @ApiOperation(value="股票列表",notes="根据参数获取对应的自选股/A股 ",response = StockSimple.class)
    @RequestMapping(value = "/list", produces = "application/json;charset=utf-8", method = RequestMethod.GET)
    public ResultVO<List<StockSimple>> list(@ApiParam(value="是否自选   0（默认）-非自选   1-自选",required=false) @RequestParam(required=false) Integer optional
            ,@ApiParam(value="股票名称或股票代码(模糊查询)",required=false) @RequestParam(required=false) String nameOrCode
            ,@ApiParam(value="预测数排序规则，0-升序  1-降序 (注：无法与priceOrderby、cpOrderby合用)",required=false) @RequestParam(required=false) Integer fnumOrderby
            ,@ApiParam(value="预测成功率排序规则，0-升序  1-降序 (注：无法与priceOrderby、cpOrderby合用))",required=false) @RequestParam(required=false) Integer frateOrderby
            ,@ApiParam(value="股价排序规则，0-升序  1-降序 (注：无法与fnumOrderby、frateOrderby合用))",required=false) @RequestParam(required=false) Integer priceOrderby
            ,@ApiParam(value="涨幅排序规则，0-升序  1-降序 (注：无法与fnumOrderby、frateOrderby合用))",required=false) @RequestParam(required=false) Integer cpOrderby
            ,@ApiParam(value="分页大小,默认10",required=false) @RequestParam(required=false) Integer size
            ,@ApiParam(value="页码 ，第一页为1") @RequestParam(required=false) Integer page) {
        ResultVO<List<StockSimple>> result = new ResultVO<List<StockSimple>>();
        result.setCode(ResultEnum.FAILED.getCode());
        result.setMsg(ResultEnum.FAILED.getMessage());
        try {
            List<StockSimple> stocks = new ArrayList<StockSimple>();
            
            List<String> codes = null;
            if (optional != null && optional == 1) {// 自选
                /* 获取用户自选股代码列表 */
                codes = stockService.selectCodeForOptional(SessionConstants.getSession());
                if (!CollectionUtils.isEmpty(codes)) {
                    stocks = stockService.selectByPage(nameOrCode, codes, priceOrderby,
                            cpOrderby, fnumOrderby, frateOrderby, size,
                            page);
                }
            }else{
                /* 获取列表 */
                stocks = stockService.selectByPage(nameOrCode, null, priceOrderby,
                        cpOrderby, fnumOrderby, frateOrderby, size,
                        page);
            }
            
            result.setData(stocks);
            result.setCode(ResultEnum.SUCCESS.getCode());
            result.setMsg(ResultEnum.SUCCESS.getMessage());
        } catch (Exception e) {
            logger.error(e.getMessage(), e);
        }

        return result;
    }
    
    @ApiOperation(value="股票详情",notes="获取股票股票现价、涨幅，市值等数据 ",response = Stock.class)
    @RequestMapping(value = "/detail/{code}", produces = "application/json;charset=utf-8", method = RequestMethod.GET)
    public ResultVO<Stock> detail(@ApiParam(value="股票代码") @PathVariable("code") String code){
        ResultVO<Stock> result = new ResultVO<Stock>();
        result.setCode(ResultEnum.FAILED.getCode());
        result.setMsg(ResultEnum.FAILED.getMessage());
        
        try {
            Stock stock = stockService.selectByCode(code);
            result.setData(stock);
            result.setCode(ResultEnum.SUCCESS.getCode());
            result.setMsg(ResultEnum.SUCCESS.getMessage());
        } catch (Exception e) {
            logger.error(e.getMessage(), e);
        }
        return result;
    }
    
    @SuppressWarnings("rawtypes")
    @ApiOperation(value="添加自选股",notes="收藏股票",response = ResultVO.class)
    @RequestMapping(value = "/collect", produces = "application/json;charset=utf-8", method = RequestMethod.GET)//TODO 应该是POST
    public ResultVO collect(@ApiParam(value="股票代码") @RequestParam("code") String code){
        ResultVO result = new ResultVO();
        result.setCode(ResultEnum.FAILED.getCode());
        result.setMsg(ResultEnum.FAILED.getMessage());
        
        try {
            Stock stock = stockService.selectByCode(code);
            if(stock == null){
                result.setMsg(Constants.MSG_NO_STOCK);
                return result;
            }
            
            Integer uid = SessionConstants.getSession();
            StockOptional optional = new StockOptional();
            optional.setUid(uid);
            optional.setStockName(stock.getName());
            optional.setStockCode(code);
            stockOptionalService.upsert(optional);
            
            result.setCode(ResultEnum.SUCCESS.getCode());
            result.setMsg(ResultEnum.SUCCESS.getMessage());
        } catch (Exception e) {
            logger.error(e.getMessage(), e);
        }
        return result;
    }
    
    @ApiOperation(value="股票k线",response = StockKVO.class)
    @RequestMapping(value = "/kList", produces = "application/json;charset=utf-8", method = RequestMethod.GET)
    public PageResultVO<List<StockKVO>> kList(@ApiParam(value="股票代码") @RequestParam("code") String code, @ApiParam(value="d日 w周 m月") @RequestParam("type") String type) {
        PageResultVO<List<StockKVO>> result = new PageResultVO<List<StockKVO>>();
        result.setCode(ResultEnum.SUCCESS.getCode());
        result.setMsg(ResultEnum.SUCCESS.getMessage());
        
        Date today = new Date();
        Date start = DateUtil.addDate(today, -150);
        if ("w".equals(type)) {
            start = DateUtils.addWeeks(today, -100);
        } else if ("m".equals(type)) {
            start = DateUtils.addMonths(today, -80);
        }

        result.setData(stockService.getStockKList(type, code, DateUtil.date2Str(start, "yyyy-MM-dd"), DateUtil.date2Str(today, "yyyy-MM-dd")));
        return result;
    }

    @SuppressWarnings("rawtypes")
    @ApiOperation(value="删除自选股",notes="删除自选股",response = ResultVO.class)
    @RequestMapping(value = "/del/{code}", produces = "application/json;charset=utf-8", method = RequestMethod.POST)
    public ResultVO del(@ApiParam(value="股票代码") @PathVariable("code") String code){
        Integer sessionId = SessionConstants.getSession();
        ResultVO vo = new ResultVO();
        if (!code.isEmpty()){
            List<StockOptional> optionals = stockOptionalService.selectOptByUid(sessionId);
            for (int i=0;i<optionals.size();i++){
                StockOptional stockOptional = optionals.get(i);
                String stockCode = stockOptional.getStockCode();
                if (code.equals(stockCode)){
                    Integer id = stockOptional.getId();
                    Integer result = stockOptionalService.delete(id);
                    if (!result.equals(1)){
                        vo.setMsg(ResultEnum.FAILED.getMessage());
                        vo.setCode(ResultEnum.FAILED.getCode());
                        return vo;
                    }
                }
            }
       }
        vo.setMsg(ResultEnum.SUCCESS.getMessage());
        vo.setCode(ResultEnum.SUCCESS.getCode());
        return vo;
    }

    @ApiOperation(value="热门股票",notes="热门股票 ",response = HotStockDTO.class)
    @RequestMapping(value = "/hot", produces = "application/json;charset=utf-8", method = RequestMethod.GET)
    public ResultVO<List<HotStockDTO>> hotStock(){
        ResultVO<List<HotStockDTO>> vo = new ResultVO<>();
        vo.setCode(ResultEnum.FAILED.getCode());
        vo.setMsg(ResultEnum.FAILED.getMessage());

        List<HotStockDTO> hotStockList = new ArrayList<>();
        List<Stock> stockList = stockService.selectHotStocks();
        for (Stock list:stockList) {
            HotStockDTO hotStockDTO = new HotStockDTO();
            hotStockDTO.setStockName(list.getName());
            hotStockDTO.setForecastCounts(list.getFnum());
            hotStockList.add(hotStockDTO);
        }
        vo.setCode(ResultEnum.SUCCESS.getCode());
        vo.setMsg(ResultEnum.SUCCESS.getMessage());
        vo.setData(hotStockList);
        return vo;
    }


//    @ApiOperation(value="股票搜索",notes="股票搜索 ",response = HotStockDTO.class)
//    @RequestMapping(value = "/search", produces = "application/json;charset=utf-8", method = RequestMethod.GET)
//    public ResultVO<> search(){
//
//    }
}
