package com.huojutech.controller.weixin;

import com.github.pagehelper.StringUtil;
import com.huojutech.constant.Constants;
import com.huojutech.constant.SessionConstants;
import com.huojutech.dto.StockForecastReleaseDTO;
import com.huojutech.entity.*;
import com.huojutech.service.*;
import com.huojutech.utils.DateUtil;
import com.huojutech.utils.ResultEnum;
import com.huojutech.utils.StringUtils;
import com.huojutech.vo.*;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.util.CollectionUtils;
import org.springframework.web.bind.annotation.*;

import javax.script.ScriptEngine;
import javax.script.ScriptEngineManager;
import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.List;
/**
 * 股票预测相关接口
 * @author zhoucong
 *
 */
@RestController
@Api(description = "股票预测相关接口")
@RequestMapping(value = "/wx/stock/*")
public class StockForecastController {
    private static Logger logger = LoggerFactory.getLogger(StockForecastController.class);
    //公式计算类
    private static ScriptEngine jse = new ScriptEngineManager().getEngineByName("JavaScript");
    
    @Autowired
    private StockService stockService;
    @Autowired
    private StockForecastService stockForecastService;
    @Autowired
    private UserService userService;
    @Autowired
    private StockForecastVoteService stockForecastVoteService;
    @Autowired
    private DiamondRecordService diamondRecordService;
    @Autowired
    private SystemConfigService systemConfigService;
    @Autowired
    private UserInviteService userInviteService;

    
    @ApiOperation(value="股票预测列表",notes="获取股票预测列表数据 ",response = StockForecastVO.class)
    @RequestMapping(value = "/forecast/list", produces = "application/json;charset=utf-8", method = RequestMethod.GET)
    public ResultVO<List<StockForecastVO>> forecastList(@ApiParam(value="股票代码") @RequestParam String code, @ApiParam(value="列表类型   0-最新；1-最热") @RequestParam Integer ltype
            ,@ApiParam(value="分页大小,默认10",required=false) @RequestParam(required=false) Integer size
            ,@ApiParam(value="页码 ，第一页为1") @RequestParam(required=false) Integer page){
        ResultVO<List<StockForecastVO>> result = new ResultVO<List<StockForecastVO>>();
        result.setCode(ResultEnum.FAILED.getCode());
        result.setMsg(ResultEnum.FAILED.getMessage());
        try {
            if (StringUtil.isEmpty(code)) {
                result.setCode(ResultEnum.PARAMS_EMPTY_FAILED.getCode());
                result.setMsg(ResultEnum.PARAMS_EMPTY_FAILED.getMessage());
                return result; 
            }
            //预测列表
            Date now = new Date();
            List<StockForecastVO> dataList = new ArrayList<StockForecastVO>();
            List<Integer> uids = new ArrayList<Integer>();
            /* 获取预测列表 */
            List<StockForecast> list = stockForecastService.selectByStockCode(code, ltype ,size, page);
            if (!CollectionUtils.isEmpty(list)) {
                for (StockForecast sf : list) {
                    uids.add(sf.getUid());
                }
                //获取用户信息
                List<User> users = userService.selectUserByUids(uids);
                
                //组装返回预测信息
                for (StockForecast sf : list) {
                    StockForecastVO vo = new StockForecastVO();
                    vo.setId(sf.getId());
                    for (User u : users) {
                        if(sf.getUid().equals(u.getId())){
                            vo.setNickname(u.getNickname());
                            vo.setAvatar(u.getAvatar());
                            vo.setFrateSuccess(u.getForecastRateSuccess());
                            break;//跳出循环
                        }
                    }
                    vo.setTarget(sf.getTargetPrice().doubleValue());
                    vo.setCycle(sf.getCycle());
                    vo.setStartDate(sf.getStartDate().getTime());
                    vo.setEndDate(sf.getEndDate().getTime());
                    //截取60长度的字符,返回前端
                    vo.setReason(StringUtils.subStrByLength(sf.getReason(), 60));
                    vo.setExpect(sf.getExpect());
                    vo.setTime(DateUtil.getWeiboTime(sf.getCrtTime()));
                    vo.setStatus(sf.getStatus());
                    if (now.before(sf.getEndDate())) {
                        vo.setDays(DateUtil.getDays(new Date(), sf.getEndDate())); 
                    }
                    dataList.add(vo);
                }
            }
            result.setData(dataList);
            result.setCode(ResultEnum.SUCCESS.getCode());
            result.setMsg(ResultEnum.SUCCESS.getMessage());
        } catch (Exception e) {
            logger.error(e.getMessage(), e);
        }
        
        return result;
    }
    
    @ApiOperation(value="股票预测详情",notes="获取股票预测详情数据 ",response = StockForecastDetailVO.class)
    @RequestMapping(value = "/forecast/detail/{id}", produces = "application/json;charset=utf-8", method = RequestMethod.GET)
    public ResultVO<StockForecastDetailVO> forecastDetail(@ApiParam(value="预测ID") @PathVariable("id") Integer id){
        ResultVO<StockForecastDetailVO> result = new ResultVO<StockForecastDetailVO>();
        result.setCode(ResultEnum.FAILED.getCode());
        result.setMsg(ResultEnum.FAILED.getMessage());
        Integer sessionId = SessionConstants.getSession();
        try {
            if (id == null) {
                result.setCode(ResultEnum.PARAMS_EMPTY_FAILED.getCode());
                result.setMsg(ResultEnum.PARAMS_EMPTY_FAILED.getMessage());
                return result; 
            }
            /*获取预测数据*/
            StockForecast sf = stockForecastService.selectById(id);
            if (sf != null) {
                StockForecastDetailVO vo = new StockForecastDetailVO();
                
                //组装预测信息
                vo.setId(sf.getId());
                vo.setSelf(sf.getUid().equals(SessionConstants.getSession()) ? 1:0);
                vo.setTarget(sf.getTargetPrice().doubleValue());
                vo.setCycle(sf.getCycle());
                vo.setStartDate(sf.getStartDate().getTime());
                vo.setEndDate(sf.getEndDate().getTime());
                vo.setReason(sf.getReason());
                vo.setExpect(sf.getExpect());
                vo.setStatus(sf.getStatus());
                if (new Date().before(sf.getEndDate())) {
                    vo.setDays(DateUtil.getDays(new Date(), sf.getEndDate())); 
                }
                
                //获取用户信息
                User user = userService.selectUserById(sf.getUid());
                if(user != null){
                    vo.setNickname(user.getNickname());
                    vo.setAvatar(user.getAvatar());
                    vo.setFrateSuccess(user.getForecastRateSuccess());
                    Integer uid = SessionConstants.getSession();
                    //判断用户是否收藏
                    if(uid != null){
                        vo.setCollected(stockForecastService.isCollected(sf.getId(), uid) ? 1:0); 
                    }else{
                        vo.setCollected(0);
                    }
                    //判断用户是否已投票
                    List<StockForecastVote> votes = stockForecastVoteService.selectByUidAndForecastId(uid, sf.getId());
                    if(!CollectionUtils.isEmpty(votes)){
                        vo.setIsAgree(votes.get(0).getVtype());
                    }else{
                        vo.setIsAgree(-1);
                    }
                }
                
                //获取股票信息
                Stock stock = stockService.selectByCode(sf.getStockCode());
                if(stock != null){
                    vo.setStockCode(stock.getCode());
                    vo.setStockName(stock.getName());
                    vo.setChangepercent(stock.getChangepercent());
                    vo.setTrade(stock.getTrade());
                    vo.setOpen(stock.getOpen());
                    vo.setPer(stock.getPer());
                    vo.setMktcap(stock.getMktcap());
                    vo.setNmc(stock.getNmc());
                    vo.setDiff(stock.getDiff());
                }
                
                //投票情况信息
                StockForecastVoteStat stat = stockForecastService.statByForecastId(sf.getId());
                if(stat != null){
                    vo.setAgreeNumber(stat.getAgreeNumber());
                    vo.setAgreeAmount(stat.getAgreeAmount());
                    vo.setAgainstNumber(stat.getAgainstNumber());
                    vo.setAgainstAmount(stat.getAgainstAmount());
                    vo.setAttendNumber(stat.getAgreeNumber() + stat.getAgainstNumber());
                }
                //投票是否成功和获得的火钻数
                List<StockForecastVote> stockForecastVotes = stockForecastVoteService.selectByUid(sessionId, sf.getId());
                if (!stockForecastVotes.isEmpty()){
                    StockForecastVote stockForecastVote = stockForecastVotes.get(0);
                    Integer status = stockForecastVote.getStatus();
                    BigDecimal harvest = stockForecastVote.getHarvest();
                    vo.setVoteStatus(status);
                    vo.setVoteSucCount(harvest);
                }

                result.setData(vo);
            }
            result.setCode(ResultEnum.SUCCESS.getCode());
            result.setMsg(ResultEnum.SUCCESS.getMessage());
        } catch (Exception e) {
            logger.error(e.getMessage(), e);
        }
        
        return result;
    }
    
    /**
     * TODO  需要事务并发处理
     */
    @ApiOperation(value="发布股票预测",notes="用于用户发布股票预测数据",response = StockForecastReleaseVO.class)
    @RequestMapping(value ="/forecast/release",produces = "application/json;charset=utf-8",method = RequestMethod.POST)
    public ResultVO<StockForecastReleaseVO> releaseForecast(@RequestBody StockForecastReleaseDTO params){

        ResultVO<StockForecastReleaseVO> result = new ResultVO<StockForecastReleaseVO>();
        result.setCode(ResultEnum.FAILED.getCode());
        result.setMsg(ResultEnum.FAILED.getMessage());
        //判断用户是否被禁用，禁用直接返回
        Integer sessionId = SessionConstants.getSession();
        User user = userService.selectUserById(sessionId);
        Integer status = user.getStatus();
        if (status.equals(-1)){
            result.setCode(ResultEnum.FORBID.getCode());
            result.setMsg(ResultEnum.FORBID.getMessage());
            return result;
        }
        //判断用户是否是第一次发布预测
        Integer firstForecast = user.getFirstForecast();//本用户
        if (firstForecast.equals(0)){
            List<UserInvite> userInvites = userInviteService.findInfoByInviteId(sessionId);
            //如果未查到邀请信息，是自己搜索进入小程序的
            if (userInvites!=null && !userInvites.isEmpty()){
                UserInvite userInvite = userInvites.get(0);
                Integer uid = userInvite.getUid();
                User user1 = userService.selectUserById(uid);
                BigDecimal amount = user1.getAmount();
                BigDecimal newAccount = amount.add(Constants.INVITE_FORECAST_VOTE_Z);
                user1.setAmount(newAccount);
                int i = userService.updateUser(user1);
                user.setFirstForecast(1);
                userService.updateUser(user);

                DiamondRecord diamondRecord = new DiamondRecord();
                diamondRecord.setUid(uid);
                diamondRecord.setName("好友预测");
                diamondRecord.setDescr("邀请好友首次预测");
                diamondRecord.setAmount(Constants.INVITE_FORECAST_VOTE_Z);
                Integer j = diamondRecordService.insertSelective(diamondRecord);
            }
        }
        //判断用户是否发布预测重复
        List<StockForecast> myForecastOn = stockForecastService.findMyForecastOn(sessionId,params.getCode(),0,params.getCycle(),new Date());
        if (myForecastOn.size()>0){
            result.setCode(ResultEnum.REPEAT_FORECAST.getCode());
            result.setMsg(ResultEnum.REPEAT_FORECAST.getMessage());
            return result;
        }

        try {
            
            if (params == null || StringUtil.isEmpty(params.getCode()) || params.getExpect() == null || StringUtil.isEmpty(params.getReason()) 
                    || params.getCycle() == null || params.getCycle() == null || params.getBet() == null || params.getBet() <= 0) {
                result.setCode(ResultEnum.PARAMS_EMPTY_FAILED.getCode());
                result.setMsg(ResultEnum.PARAMS_EMPTY_FAILED.getMessage());
                return result;
            }
            
            Integer uid = SessionConstants.getSession();
            
            //获取对应的股票
            Stock stock = stockService.selectByCode(params.getCode());
            if(stock == null){
                result.setMsg(Constants.MSG_NO_STOCK);
                return result;
            }
            if (stock.getOpen()==0.0){
                result.setMsg(Constants.MSG_NO_STOCK);
                return result;
            }
            
            /*
             * 扣除对应火钻数
             */
            if (userService.updateAmountByUid(uid, -params.getBet()) <= 0) {
                result.setMsg(Constants.MSG_DIAMOND_NOT_ENOUGH);
                return result;
            }
            
            //火钻使用记录
            DiamondRecord record = new DiamondRecord();
            record.setUid(uid);
            //发布预测，火钻数记录为负数
            record.setAmount(new BigDecimal(-params.getBet()));
            record.setName("股票预测-发布");
            record.setDescr("发布 针对 股票["+stock.getName()+"]["+stock.getCode()+"]的预测");
            diamondRecordService.insertSelective(record);
            
            /* 组装股票预测记录 */
            StockForecast cast = new StockForecast();
            cast.setUid(uid);
            cast.setStockCode(stock.getCode());
            cast.setStockName(stock.getName());
            cast.setExpect(params.getExpect());
            cast.setReason(params.getReason());//TODO cong 需要对该字段做相应校验
            cast.setBet(new BigDecimal(params.getBet()));
            cast.setCurrentPrice(new BigDecimal(stock.getTrade()));
            cast.setTargetPrice(new BigDecimal(params.getTarget()));
            Integer cycle = params.getCycle();
            cast.setCycle(cycle);
            //计算开始时间和结束时间
            Calendar c = Calendar.getInstance();
            cast.setStartDate(c.getTime());
            switch (cycle) {
            case 1: c.add(Calendar.DAY_OF_YEAR, 7); break;
            case 2: c.add(Calendar.MONTH, 1); break;
            case 3: c.add(Calendar.MONTH, 2); break;
            case 4: c.add(Calendar.MONTH, 3); break;
            case 5: c.add(Calendar.MONTH, 6); break;
            case 6: c.add(Calendar.MONTH, 12); break;
            default: break;
            }
            cast.setEndDate(c.getTime());
            /*插入数据库*/
            stockForecastService.insertSelective(cast);
            
            /*组装返回参数集*/
            StockForecastReleaseVO vo = new StockForecastReleaseVO();
            vo.setId(cast.getId());
            vo.setName(cast.getStockName());
            vo.setCode(cast.getStockCode());
            vo.setExpect(cast.getExpect());
            vo.setTarget(cast.getTargetPrice().doubleValue());
            vo.setCycle(cast.getCycle());
            vo.setStartDate(cast.getStartDate().getTime());
            vo.setEndDate(cast.getEndDate().getTime());
            vo.setBet(cast.getBet().doubleValue());
            
            //获取预测火钻生成公式
            List<SystemConfig> configs = systemConfigService.selectByKey(Constants.CONFIG_FORECAST_FORMULA);
            if(!CollectionUtils.isEmpty(configs) && StringUtil.isNotEmpty(configs.get(0).getCfgVal())){
                String formula = configs.get(0).getCfgVal();
                //计算价位变化幅度
                Double changepercent = Math.abs(cast.getCurrentPrice().subtract(cast.getTargetPrice()).divide(cast.getCurrentPrice(), 6, BigDecimal.ROUND_HALF_UP).doubleValue());
                //TODO cong vbet,最好把这个计算公式抽出来，以后可能会变
                formula = formula.replace("#bet#", cast.getBet()+"").replace("#changepercent#", String.valueOf(changepercent)).replace("#vbet#", "0");
                vo.setHarvest(Double.valueOf(String.format("%.6f", jse.eval(formula))));
            }else{
                vo.setHarvest(cast.getBet().doubleValue());  
            }
            
            result.setData(vo);
            result.setCode(ResultEnum.SUCCESS.getCode());
            result.setMsg(ResultEnum.SUCCESS.getMessage());
            
            /*
             * 股票预测数+1
             */
            stockService.updateNumberByCode(stock.getCode(), 1, 0);
            /*
             * 用户预测数+1
             */
            userService.updateNumberById(uid, 1, 0, 0,0);
        } catch (Exception e) {
            logger.error(e.getMessage(), e);
        }
        
        return result;
    }
    
    /**
     * TODO  需要事务并发处理
     */
    @ApiOperation(value="投票",notes="对股票预测进行投票 ",response = StockForecastVoteVO.class)
    @RequestMapping(value = "/forecast/vote", produces = "application/json;charset=utf-8", method = RequestMethod.GET) //TODO POST
    public ResultVO<StockForecastVoteVO> vote(@ApiParam(value="预测ID") @RequestParam("forecastId") Integer forecastId, @ApiParam(value="是否赞同   0-赞同  1-反对") @RequestParam(value="isAgree") Integer isAgree
            , @ApiParam(value="下注火钻数") @RequestParam(value="bet") Double bet){
        ResultVO<StockForecastVoteVO> result = new ResultVO<StockForecastVoteVO>();
        result.setCode(ResultEnum.FAILED.getCode());
        result.setMsg(ResultEnum.FAILED.getMessage());

        //判断用户是否被禁用
        Integer sessionId = SessionConstants.getSession();
        User user = userService.selectUserById(sessionId);
        Integer status = user.getStatus();
        if (status.equals(-1)){
            result.setCode(ResultEnum.FORBID.getCode());
            result.setMsg(ResultEnum.FORBID.getMessage());
            return result;
        }
        //判断用户是否是第一次投票
        Integer firstVote = user.getFirstVote();
        if (firstVote.equals(0)){
            List<UserInvite> userInvites = userInviteService.findInfoByInviteId(sessionId);
            //如果未查到邀请信息，是自己搜索进入小程序的
            if (userInvites!=null && !userInvites.isEmpty()){
                UserInvite userInvite = userInvites.get(0);
                Integer uid = userInvite.getUid();//邀请自己的人的id
                User user1 = userService.selectUserById(uid);
                BigDecimal amount = user1.getAmount();
                BigDecimal newAccount = amount.add(Constants.INVITE_FORECAST_VOTE_Z);
                user1.setAmount(newAccount);
                int i = userService.updateUser(user1);

                user.setFirstVote(1);
                userService.updateUser(user);
                DiamondRecord diamondRecord = new DiamondRecord();
                diamondRecord.setUid(uid);
                diamondRecord.setName("好友投票");
                diamondRecord.setDescr("邀请好友首次投票");
                diamondRecord.setAmount(Constants.INVITE_FORECAST_VOTE_Z);
                Integer j = diamondRecordService.insertSelective(diamondRecord);
                if (i==0){
                    result.setCode(ResultEnum.INFO_UPDATE_FAILED.getCode());
                    result.setMsg(ResultEnum.INFO_UPDATE_FAILED.getMessage());
                    return result;
                }
                if (j==0){
                    result.setCode(ResultEnum.INFO_SAVE_FAILED.getCode());
                    result.setMsg(ResultEnum.INFO_SAVE_FAILED.getMessage());
                    return result;
                }
            }
        }
        try {
            if(forecastId == null && isAgree == null && bet == null){
                result.setCode(ResultEnum.PARAMS_EMPTY_FAILED.getCode());
                result.setMsg(ResultEnum.PARAMS_EMPTY_FAILED.getMessage());
                return result;
            }
            
            Integer uid = SessionConstants.getSession();
            // 获取该用户投票记录
            Integer total = stockForecastVoteService.countByUidAndForecastId(uid, forecastId);
            if (total > 0) {
                result.setMsg(Constants.MSG_NO_REPEATVOTE_FAILED);
                return result;
            }
            //获取预测数据
            StockForecast cast = stockForecastService.selectById(forecastId);
            if (cast == null) {
                result.setMsg(Constants.MSG_NO_STOCK_FORECAST);
                return result;
            }
            if (cast.getStatus() != 0) {
                result.setMsg(Constants.MSG_FORECAST_OVER);
                return result;
            }
            if(uid.equals(cast.getUid())){
                result.setMsg(Constants.MSG_VOTE_FOR_SELF);
                return result;
            }
            
            if (userService.updateAmountByUid(uid, -bet) <= 0) {
                result.setMsg(Constants.MSG_DIAMOND_NOT_ENOUGH);
                return result;
            }
            //火钻使用记录
            DiamondRecord record = new DiamondRecord();
            record.setUid(uid);
            //发布预测，火钻数记录为负数
            record.setAmount(new BigDecimal(-bet));
            record.setName("股票预测-投票");
            record.setDescr("投票 对股票预测["+cast.getId()+"]投了"+(isAgree == 0 ? "赞同":"反对") +"票");
            diamondRecordService.insertSelective(record);
            /* 添加投票记录 */
            StockForecastVote vote = new StockForecastVote();
            vote.setUid(uid);
            vote.setForecastId(cast.getId());
            vote.setVtype(isAgree);
            vote.setBet(new BigDecimal(bet));
            int row = stockForecastVoteService.upsert(vote);
            if (row > 0) {
                /* 组装返回结果集 */
                StockForecastVoteVO vo = new StockForecastVoteVO();
                vo.setId(vote.getId());
                vo.setIsAgree(vote.getVtype());
                vo.setBet(vote.getBet().doubleValue());
                //获取预测火钻生成公式
                List<SystemConfig> configs = systemConfigService.selectByKey(Constants.CONFIG_FORECAST_VOTE_FORMULA);
                if(!CollectionUtils.isEmpty(configs) && StringUtil.isNotEmpty(configs.get(0).getCfgVal())){
                    String formula = configs.get(0).getCfgVal();
                    //计算价位变化幅度
                    Double changepercent = Math.abs(cast.getCurrentPrice().subtract(cast.getTargetPrice()).divide(cast.getCurrentPrice(), 6, BigDecimal.ROUND_HALF_UP).doubleValue());
                    //TODO cong 记录下当前的计算公式
                    formula = formula.replace("#bet#", vote.getBet()+"").replace("#changepercent#", String.valueOf(changepercent));
                    Double havestValue = Double.valueOf(String.format("%.6f", jse.eval(formula)));
                    vo.setHarvest(havestValue);

                }else{
                    vo.setHarvest(vote.getBet().doubleValue());
                }
                
                vo.setName(cast.getStockName());
                vo.setCode(cast.getStockCode());
                vo.setExpect(cast.getExpect());
                vo.setTarget(cast.getTargetPrice().doubleValue());
                vo.setCycle(cast.getCycle());
                vo.setStartDate(cast.getStartDate().getTime());
                vo.setEndDate(cast.getEndDate().getTime());
                
                result.setData(vo);
                result.setCode(ResultEnum.SUCCESS.getCode());
                result.setMsg(ResultEnum.SUCCESS.getMessage());
                
                /* 预测 投票数+1 */
                stockForecastService.updateNumberById(cast.getId(), 1);
                /* 股票 投票数+1 */
                stockService.updateNumberByCode(cast.getStockCode(), 0, 1);
                /* 被投票用户被投票数+1 */
                userService.updateNumberById(cast.getUid(), 0, 1, 0,0);
                //投票用户，投票数+1
                userService.updateNumberById(uid, 0, 0, 0,1);

            }
        } catch (Exception e) {
            logger.error(e.getMessage(), e);
        }
        return result;
    }


}
