package com.huojutech.controller.weixin;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.github.pagehelper.PageHelper;
import com.huojutech.constant.SessionConstants;
import com.huojutech.dto.*;
import com.huojutech.entity.*;
import com.huojutech.service.*;
import com.huojutech.utils.AesCbcUtil;
import com.huojutech.utils.ResultEnum;
import com.huojutech.vo.PageResultVO;
import com.huojutech.vo.ResultVO;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.data.redis.core.ValueOperations;
import org.springframework.web.bind.annotation.*;

import java.math.BigDecimal;
import java.text.DecimalFormat;
import java.text.ParseException;
import java.util.ArrayList;
import java.util.List;

/**
 * Created by Wangzw on 2018\4\16 0016.
 */
@RestController
@Api(description = "用户中心接口")
@RequestMapping("/wx/userCenter")
public class UserCenterController {

    private static Logger logger = LoggerFactory.getLogger(UserCenterController.class);

    @Autowired
    private UserConcernService userConcernService;

    @Autowired
    private StockOptionalService stockOptionalService;

    @Autowired
    private UserService userService;

    @Autowired
    private ForecastService forecastService;

    @Autowired
    private UserInviteService userInviteService;

    @Autowired
    private StockForecastVoteService stockForecastVoteService;

    @Autowired
    private StockForecastService stockForecastService;

    @Autowired
    private RedisTemplate redisTemplate;

    @RequestMapping(value = "/info", produces = "application/json;charset=utf-8", method = RequestMethod.GET)
    @ApiOperation(value = "查询用户中心所有立即展示的数据", notes = "通过用户id查询", response = CountDTO.class)
    public ResultVO<CountDTO> myUserCenterNum() throws Exception {
        Integer uid = SessionConstants.getSession();
        Integer concernCounts =  userConcernService.selectConcernCounts(uid);//用户关注数量
        Integer fansCounts = userConcernService.selectFansCounts(uid);//用户粉丝数量
        User user = userService.selectUserById(uid);
        String nickname = user.getNickname();
        String avatar = user.getAvatar();

        BigDecimal amount = user.getAmount();//拥有火钻数量
        String stringAmount = amount.toString();

        CountDTO countDTO = new CountDTO();
        countDTO.setUserConcernNum(concernCounts);
        countDTO.setFansNum(fansCounts);
        countDTO.setDiamondNum(stringAmount);
        countDTO.setAvatar(avatar);
        countDTO.setNickName(nickname);

        ResultVO<CountDTO> vo = new ResultVO<>();
        vo.setData(countDTO);
        vo.setCode(ResultEnum.SUCCESS.getCode());
        vo.setMsg(ResultEnum.SUCCESS.getMessage());
        return vo;
    }

    @RequestMapping(value = "/fans/List", produces = "application/json;charset=utf-8", method = RequestMethod.GET)
    @ApiOperation(value = "查询本用户的粉丝列表", notes = "通过用户id查询", response = CountDTO.class)
    public PageResultVO<List<SeerListDTO>> fans(
            @ApiParam(value="分页大小,默认10",required=false) @RequestParam(required=false,defaultValue="10") Integer size,
            @ApiParam(value="页码 ，第一页为1") @RequestParam(required=false,defaultValue="1") Integer page) throws Exception {
        Integer uid = SessionConstants.getSession();
        List<SeerListDTO> seerListDTOS = new ArrayList<>();
        PageHelper.startPage(page,size);
        List<UserConcern> fansList = userConcernService.selectFansList(uid);
        getPerson(seerListDTOS,fansList);//获取粉丝抽取
        PageResultVO<List<SeerListDTO>> vo = new PageResultVO<>();
        vo.setData(seerListDTOS);

        vo.setCode(ResultEnum.SUCCESS.getCode());
        vo.setMsg(ResultEnum.SUCCESS.getMessage());
        return vo;
    }

    @RequestMapping(value = "/concerns/List", produces = "application/json;charset=utf-8", method = RequestMethod.GET)
    @ApiOperation(value = "查询本用户的粉丝列表", notes = "通过用户id查询", response = CountDTO.class)
    public PageResultVO<List<SeerListDTO>> concern(
            @ApiParam(value="分页大小,默认10",required=false) @RequestParam(required=false,defaultValue="10") Integer size,
            @ApiParam(value="页码 ，第一页为1") @RequestParam(required=false,defaultValue="1") Integer page) throws Exception {
        Integer uid = SessionConstants.getSession();
        List<SeerListDTO> seerListDTOS = new ArrayList<>();
        PageHelper.startPage(page,size);
        List<UserConcern> concernList = userConcernService.selectConcernList(uid);
        getPerson(seerListDTOS, concernList);//获取关注抽取
        PageResultVO<List<SeerListDTO>> vo = new PageResultVO<>();
        vo.setData(seerListDTOS);

        vo.setCode(ResultEnum.SUCCESS.getCode());
        vo.setMsg(ResultEnum.SUCCESS.getMessage());
        return vo;
    }

    /**
     * 获取用户粉丝和关注抽取方法
     * @param seerListDTOS
     * @param concernList
     */
    private void getPerson(List<SeerListDTO> seerListDTOS, List<UserConcern> concernList) {
        for (UserConcern list:concernList) {
            SeerListDTO seerListDTO = new SeerListDTO();
            Integer uid1 = list.getUid();
            User user = userService.selectUserById(uid1);
            String nickname = user.getNickname();//昵称
            String avatar = user.getAvatar();//头像
            Double forecastRateSuccess = user.getForecastRateSuccess();

            DecimalFormat df = new DecimalFormat("0.00%");
            String forecastSuc = df.format(forecastRateSuccess);//成功率

            Integer forecastNum = forecastService.findForecastNumByUid(uid1);//预测数量
            Integer concernCounts = userConcernService.selectFansCounts(uid1);//别人关注他数量

            seerListDTO.setId(uid1);
            seerListDTO.setAvatar(avatar);
            seerListDTO.setNickname(nickname);
            seerListDTO.setForecastSuc(forecastSuc);
            seerListDTO.setConcernNum(concernCounts);
            seerListDTO.setForecastCounts(forecastNum);

            seerListDTOS.add(seerListDTO);
        }
    }

    @SuppressWarnings("rawtypes")
    @RequestMapping(value = "/sign/info", produces = "application/json;charset=utf-8", method = RequestMethod.POST)
    @ApiOperation(value = "用户授权时获取用户信息", notes = "用户授权时获取用户信息")
    public ResultVO signInfo(@RequestBody UserInfoDTO userInfoDTO) {
        Integer sessionId = SessionConstants.getSession();
        ResultVO vo = new ResultVO();
        if (userInfoDTO != null) {
            String nickname = userInfoDTO.getNickname();
            String avatar = userInfoDTO.getAvatar();
            User user = new User();
            user.setId(sessionId);
            user.setNickname(nickname);
            user.setAvatar(avatar);

            ValueOperations valueOperations = redisTemplate.opsForValue();
            String sessionKey = (String) valueOperations.get("sessionKey");
            String result = null;
            try {
                result = AesCbcUtil.decrypt(userInfoDTO.getEncryptedData(),sessionKey, userInfoDTO.getIv(), "UTF-8");
            } catch (Exception e) {
                logger.info(e.getMessage(), e);
            }
            if (null != result && result.length() > 0) {
                JSONObject userInfoJSON = JSON.parseObject(result);
                String unionId = (String) userInfoJSON.get("unionId");
                user.setWxUnionid(unionId);
            }
            userService.updateUser(user);
        }

        vo.setMsg(ResultEnum.SUCCESS.getMessage());
        vo.setCode(ResultEnum.SUCCESS.getCode());
        return vo;
    }
    @SuppressWarnings("rawtypes")
    @RequestMapping(value = "/invite/{id}", produces = "application/json;charset=utf-8", method = RequestMethod.POST)
    @ApiOperation(value = "邀请用户", notes = "邀请用户")
    public ResultVO invite(@ApiParam(value="邀请者的id") @PathVariable(value="id") Integer id) {
        Integer sessionId = SessionConstants.getSession();
        ResultVO vo = new ResultVO();
        if (id!=null){
            //如果用户不存在，是盗刷连接
            User user = userService.selectUserById(id);
            if (user==null){
                vo.setMsg(ResultEnum.FAILED.getMessage());
                vo.setCode(ResultEnum.FAILED.getCode());
                return vo;
            }
            //如果id是自己，自己不能邀请自己
            if (sessionId.equals(id)){
                vo.setMsg(ResultEnum.FAILED.getMessage());
                vo.setCode(ResultEnum.FAILED.getCode());
                return vo;
            }
            //如果自己已经被邀请过了，此邀请不能不算数
            List<UserInvite> inviteUser = userInviteService.findInfoByInviteId(sessionId);
            if (inviteUser.size()>0){
                vo.setMsg(ResultEnum.REPEAT_INVITE.getMessage());
                vo.setCode(ResultEnum.REPEAT_INVITE.getCode());
                return vo;
            }
            UserInvite userInvite = new UserInvite();
            userInvite.setUid(id);
            userInvite.setInviteId(sessionId);
            userInviteService.insertUser(userInvite);
        }
        vo.setMsg(ResultEnum.SUCCESS.getMessage());
        vo.setCode(ResultEnum.SUCCESS.getCode());
        return vo;
    }

    @SuppressWarnings({ "unchecked" })
    @RequestMapping(value = "/score", produces = "application/json;charset=utf-8", method = RequestMethod.GET)
    @ApiOperation(value = "我的战绩接口", notes = "我的战绩")
    public ResultVO<ScoreDTO> score() {
        Integer sessionId = SessionConstants.getSession();
        ResultVO<ScoreDTO> vo = new ResultVO<>();
        vo.setCode(ResultEnum.FAILED.getCode());
        vo.setMsg(ResultEnum.FAILED.getMessage());

        ScoreDTO scoreDTO = new ScoreDTO();
        ScoreInfoDTO scoreInfoDTO = new ScoreInfoDTO();

        List<ScoreForecastInfoDTO> scoreForecastInfoDTOS = new ArrayList<>();
        List<ScoreVoteInfoDTO> scoreVoteInfoDTOS = new ArrayList<>();

        User user = userService.selectUserById(sessionId);
        Integer fnum = user.getFnum();//累计预测
        Integer voteNum = user.getVoteNum();//累计投票
        Double forecastRateSuccess = user.getForecastRateSuccess();
        Double voteRateSuccess = user.getVoteRateSuccess();
        DecimalFormat df = new DecimalFormat("0.00%");
        String forecastSuc = df.format(forecastRateSuccess);//预测成功率
        String voteRateSuc = df.format(voteRateSuccess);//投票成功率
        List<StockForecast> seerForecast = forecastService.findSeerForecast(sessionId);


        BigDecimal forecastBet = new BigDecimal(0);
        BigDecimal forecastHarvest = new BigDecimal(0);
        if (!seerForecast.isEmpty()){
            for(int i=0;i<seerForecast.size();i++){
                StockForecast stockForecast = seerForecast.get(i);
                BigDecimal bet = stockForecast.getBet();
                forecastBet = forecastBet.add(bet);

                BigDecimal harvest = stockForecast.getHarvest();
                forecastHarvest = forecastHarvest.add(harvest);
                //预测列表
                ScoreForecastInfoDTO scoreForecastInfoDTO = new ScoreForecastInfoDTO();
                scoreForecastInfoDTO.setForecastId(stockForecast.getId());
                scoreForecastInfoDTO.setBet(stockForecast.getBet());
                scoreForecastInfoDTO.setCrtTime(stockForecast.getCrtTime());
                scoreForecastInfoDTO.setExpect(stockForecast.getExpect());
                scoreForecastInfoDTO.setHarvest(stockForecast.getHarvest());
                scoreForecastInfoDTO.setStatus(stockForecast.getStatus());
                scoreForecastInfoDTO.setStockName(stockForecast.getStockName());
                scoreForecastInfoDTO.setTargetPrice(stockForecast.getTargetPrice());
                scoreForecastInfoDTO.setOpt_type("预测");
                scoreForecastInfoDTOS.add(scoreForecastInfoDTO);
            }
        }

        BigDecimal voteBet = new BigDecimal(0);
        BigDecimal voteHarvest = new BigDecimal(0);
        List<StockForecastVote> forecastVoteList = stockForecastVoteService.findVoteByUid(sessionId);
        if (!forecastVoteList.isEmpty()){
            for(int j=0;j<forecastVoteList.size();j++){
                StockForecastVote stockForecastVote = forecastVoteList.get(j);
                BigDecimal bet = stockForecastVote.getBet();
                voteBet = voteBet.add(bet);

                BigDecimal harvest = stockForecastVote.getHarvest();
                voteHarvest = voteHarvest.add(harvest);
                //投票列表
                Integer forecastId = stockForecastVote.getForecastId();
                StockForecast stockForecast = stockForecastService.selectById(forecastId);
                ScoreVoteInfoDTO scoreVoteInfoDTO = new ScoreVoteInfoDTO();
                scoreVoteInfoDTO.setBet(bet);
                scoreVoteInfoDTO.setCrtTime(stockForecastVote.getCrtTime());
                scoreVoteInfoDTO.setExpect(stockForecast.getExpect());
                scoreVoteInfoDTO.setHarvest(stockForecastVote.getHarvest());
                scoreVoteInfoDTO.setStatus(stockForecast.getStatus());
                scoreVoteInfoDTO.setForecastId(forecastId);
                scoreVoteInfoDTO.setStockName(stockForecast.getStockName());
                scoreVoteInfoDTO.setTargetPrice(stockForecast.getTargetPrice());
                scoreVoteInfoDTO.setOpt_type("投票");
                scoreVoteInfoDTOS.add(scoreVoteInfoDTO);
            }
        }
        scoreInfoDTO.setScoreForecastInfoDTO(scoreForecastInfoDTOS);
        scoreInfoDTO.setScoreVoteInfoDTO(scoreVoteInfoDTOS);

        //累计中下注额
        BigDecimal bet = forecastBet.add(voteBet);
        //累计赢取火钻
        BigDecimal harvest = forecastHarvest.add(voteHarvest);

        scoreDTO.setBet(bet);
        scoreDTO.setFnum(fnum);
        scoreDTO.setForecastSuc(forecastSuc);
        scoreDTO.setHarvest(harvest);
        scoreDTO.setVoteNum(voteNum);
        scoreDTO.setVoteSuc(voteRateSuc);
        scoreDTO.setScoreInfo(scoreInfoDTO);

        vo.setCode(ResultEnum.SUCCESS.getCode());
        vo.setMsg(ResultEnum.SUCCESS.getMessage());
        vo.setData(scoreDTO);
        return vo;
    }

    @RequestMapping(value = "/recommend", produces = "application/json;charset=utf-8", method = RequestMethod.GET)
    @ApiOperation(value = "查询预言家列表", notes = "预言家列表", response = SeerListDTO.class)
    public ResultVO<List<RecommendDTO>> recommend() {
        ResultVO<List<RecommendDTO>> vo  = new ResultVO<>();
        vo.setMsg(ResultEnum.FAILED.getMessage());
        vo.setCode(ResultEnum.FAILED.getCode());

        List<RecommendDTO> recommendDTOS = new ArrayList<>();

        List<User> userList = userService.selectRecommendUser();
        for (User list:userList) {
            RecommendDTO recommendDTO = new RecommendDTO();
            recommendDTO.setId(list.getId());
            recommendDTO.setNickname(list.getNickname());
            recommendDTO.setAvatar(list.getAvatar());
            Double forecastRateSuccess = list.getForecastRateSuccess();
            DecimalFormat df = new DecimalFormat("0.00%");
            String forecastSuc = df.format(forecastRateSuccess);//成功率
            recommendDTO.setForecastSuc(forecastSuc);
            recommendDTOS.add(recommendDTO);
        }
        vo.setData(recommendDTOS);
        vo.setMsg(ResultEnum.SUCCESS.getMessage());
        vo.setCode(ResultEnum.SUCCESS.getCode());
        return vo;
    }

}
