package com.huojutech.controller.weixin;


import com.github.pagehelper.PageHelper;
import com.github.pagehelper.StringUtil;
import com.huojutech.constant.Constants;
import com.huojutech.constant.SessionConstants;
import com.huojutech.dto.VoteDTO;
import com.huojutech.entity.StockForecast;
import com.huojutech.entity.StockForecastVote;
import com.huojutech.entity.SystemConfig;
import com.huojutech.entity.User;
import com.huojutech.service.StockForecastService;
import com.huojutech.service.StockForecastVoteService;
import com.huojutech.service.SystemConfigService;
import com.huojutech.service.UserService;
import com.huojutech.utils.DateUtil;
import com.huojutech.utils.ResultEnum;
import com.huojutech.vo.PageResultVO;
import com.huojutech.vo.ResultVO;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.util.CollectionUtils;
import org.springframework.web.bind.annotation.*;

import javax.script.ScriptEngine;
import javax.script.ScriptEngineManager;
import javax.script.ScriptException;
import java.math.BigDecimal;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

/**
 * Created by Administrator on 2018/4/17.
 */
@RestController
@Api(description = "我的投票接口")
@RequestMapping("/wx/vote")
@SuppressWarnings({ "rawtypes", "unchecked" })
public class VoteController {

    @Autowired
    private StockForecastService stockForecastService;

    @Autowired
    private UserService userService;

    @Autowired
    private SystemConfigService systemConfigService;

    @Autowired
    private StockForecastVoteService stockForecastVoteService;

    private static ScriptEngine jse = new ScriptEngineManager().getEngineByName("JavaScript");

    @RequestMapping(value = "/list", produces = "application/json;charset=utf-8", method = RequestMethod.GET)
    @ApiOperation(value="查询用户投票的数量",notes="根据传来的用户id，和预测状态status，0预测中,1已经结束",response = VoteDTO.class)
    public PageResultVO<VoteDTO> myVote(@ApiParam(value = "预测状态0:预测中，1:已经结束") @RequestParam Integer status,
                                              @ApiParam(value="分页大小,默认10",required=false) @RequestParam(required=false,defaultValue="10") Integer size,
                                              @ApiParam(value="页码 ，第一页为1") @RequestParam(required=false,defaultValue="1") Integer page) throws ParseException, ScriptException {
        Integer uid = SessionConstants.getSession();
        List<VoteDTO> myVoteDTOList = new ArrayList<>();
        List<StockForecast> list =null;
        PageHelper.startPage(page,size);
        if (status.equals(0)) {
            list = stockForecastService.findMyVoteOn(uid, status);
        }else {
            List<Integer> listStatus = new ArrayList<>();
            listStatus.add(1);
            listStatus.add(2);
            list =stockForecastService.findMyVoteOff(uid,listStatus);
        }

        for (StockForecast forecastInfo : list) {
            VoteDTO voteDTO = new VoteDTO();
            Integer uid1 = forecastInfo.getUid();
            if (uid1.equals(uid)){//如果是自己预测的股票，不显示在投票列表
                continue;
            }
            User user = userService.selectUserById(uid1);

            Date startDate = forecastInfo.getStartDate();
            Date endDate = forecastInfo.getEndDate();

            SimpleDateFormat format = new SimpleDateFormat("yyyy.MM.dd");
            String startStringDate = format.format(startDate);
            String endStringDate = format.format(endDate);
            String timeZone = startStringDate+"-"+endStringDate;

            Date todayTime = new Date();
            String format1 = format.format(todayTime);
            Date parse =null;
            Date parse2=null;
            parse = format.parse(format1);//当前时间
            parse2 = format.parse(endStringDate);//预测结束时间
            int restTime = DateUtil.diffDate(parse, parse2);

            //如果预测结束，直接返回数据库中的火钻
            List<StockForecastVote> stockForecastVotes = stockForecastVoteService.selectByUid(uid, forecastInfo.getId());
            if (!status.equals(0)){
                if (!stockForecastVotes.isEmpty()){
                    StockForecastVote stockForecastVote = stockForecastVotes.get(0);
                    BigDecimal bet = stockForecastVote.getBet();
                    BigDecimal harvest = stockForecastVote.getHarvest();
                    voteDTO.setHarvest(harvest);
                }
            }else {
                //预测投票进行中，返回如果成功规则计算出的应得火钻
                BigDecimal bet = forecastInfo.getBet();
                BigDecimal currentPrice = forecastInfo.getCurrentPrice();
                BigDecimal targetPrice = forecastInfo.getTargetPrice();
                BigDecimal subtract = targetPrice.subtract(currentPrice);
                BigDecimal divide = new BigDecimal(0);
                int i2 = currentPrice.compareTo(BigDecimal.ZERO);
                if (i2!=0){
                    int i = subtract.compareTo(BigDecimal.ZERO);
                    if (i==-1){
                        String substring = subtract.toString().substring(1);
                        subtract = new BigDecimal(substring);
                    }
                    divide = subtract.divide(currentPrice,2,BigDecimal.ROUND_HALF_UP);
                }else {
                    divide =subtract;
                }

                //#bet#*(1+#changepercent#)
                List<SystemConfig> systemConfigs = systemConfigService.selectByKey(Constants.CONFIG_FORECAST_VOTE_FORMULA);
                if(!CollectionUtils.isEmpty(systemConfigs) && StringUtil.isNotEmpty(systemConfigs.get(0).getCfgVal())){
                    SystemConfig systemConfig = systemConfigs.get(0);
                    String cfgVal = systemConfig.getCfgVal();
                    cfgVal = cfgVal.replace("#bet#", bet+"").replace("#changepercent#", divide+"");
                    String have = String.format("%.6f", jse.eval(cfgVal));
                    BigDecimal harvest = new BigDecimal(have);
                    harvest = harvest.setScale(6, BigDecimal.ROUND_HALF_UP);
                    voteDTO.setHarvest(harvest);

                    if (!stockForecastVotes.isEmpty()){
                        StockForecastVote stockForecastVote = stockForecastVotes.get(0);
                        BigDecimal bet1 = stockForecastVote.getBet();
                        voteDTO.setBet(bet1);
                    }
                }
            }
            voteDTO.setName(user.getNickname());
            voteDTO.setStockName(forecastInfo.getStockName());
            voteDTO.setExpect(forecastInfo.getExpect());
            voteDTO.setTargetPrice(forecastInfo.getTargetPrice());
            voteDTO.setCycle(forecastInfo.getCycle());
            voteDTO.setMyStatus(forecastInfo.getStatus());
            voteDTO.setTimeZone(timeZone);
            voteDTO.setRestTime(restTime);
            voteDTO.setForecastId(forecastInfo.getId());

            myVoteDTOList.add(voteDTO);
        }
        PageResultVO vo = new PageResultVO();
        vo.setData(myVoteDTOList);
        vo.setCode(ResultEnum.SUCCESS.getCode());
        vo.setMsg(ResultEnum.SUCCESS.getMessage());
        return vo;
    }

    @RequestMapping(value = "/frofile/{forecastId}", produces = "application/json;charset=utf-8", method = RequestMethod.POST)
    @ApiOperation(value = "用户点击投票，重新计算预测收益", notes = "点击投票，重新计算预测收益")
    public ResultVO frofile(@ApiParam(value = "预测id") @PathVariable Integer forecastId) throws ScriptException {
        Integer sessionId = SessionConstants.getSession();
        ResultVO vo  = new ResultVO();
        vo.setCode(ResultEnum.FAILED.getCode());
        vo.setMsg(ResultEnum.FAILED.getMessage());

        StockForecast stockForecast = stockForecastService.selectById(forecastId);
        Integer vnum = stockForecast.getVnum();
        List<SystemConfig> configs = systemConfigService.selectByKey(Constants.CONFIG_FORECAST_FORMULA);

        if(!CollectionUtils.isEmpty(configs) && StringUtil.isNotEmpty(configs.get(0).getCfgVal())){
            String formula = configs.get(0).getCfgVal();
            //计算价位变化幅度
            Double changepercent = Math.abs(stockForecast.getCurrentPrice().subtract(stockForecast.getTargetPrice()).divide(stockForecast.getCurrentPrice(), 6, BigDecimal.ROUND_HALF_UP).doubleValue());
            String formula1 = formula.replace("#bet#", stockForecast.getBet()+"").replace("#changepercent#", String.valueOf(changepercent)).replace("#vbet#", vnum+"");
            String formatA = String.format("%.6f", jse.eval(formula1));
            String formula2 = formula.replace("#bet#", stockForecast.getBet()+"").replace("#changepercent#", String.valueOf(changepercent)).replace("#vbet#", "0");
            String formatB = String.format("%.6f", jse.eval(formula2));
            BigDecimal subtract = new BigDecimal(formatA).subtract(new BigDecimal(formatB));
            stockForecastService.updateVoteHarvestById(forecastId,subtract);
        }
        vo.setCode(ResultEnum.SUCCESS.getCode());
        vo.setMsg(ResultEnum.SUCCESS.getMessage());
        return vo;
    }
}

