package com.huojutech.dao;

import com.huojutech.entity.StockForecast;
import com.huojutech.entity.StockForecastVote;
import org.apache.ibatis.annotations.Param;

import java.util.List;

public interface StockForecastTaskMapper {
   /**
    * 统计股票总记录数 
    * @return
    */
   Integer countStock();
   
   /**
    * 获取股票代码
    * @param size           分页大小
    * @param offset         
    * @return
    */
   List<String> selectStockCodes(@Param(value="size") Integer size, @Param(value="offset") Integer offset);
   
   /**
    * 更新股票预测成功率
    * @param code       股票代码
    * @return
    */
   int updateFrateForStock(@Param(value="code") String code);
   
   /**
    * 获取用户总记录数
    * @return
    */
   Integer countUsers();
   
   /**
    * 获取用户id
    * @param size       分页大小
    * @param offset
    * @return
    */
   List<Integer> selectUids(@Param(value="size") Integer size, @Param(value="offset") Integer offset);
   
   /**
    * 更新用户预测成功率
    * @param uid        用户id
    * @return
    */
   int updateFrateForUser(@Param(value="uid") Integer uid);
   
   /**
    * 获取配置信息
    * @param key
    * @return
    */
   String selectConfigByKey(@Param(value="key") String key);
   
   /**
    * 统计进行中的预测记录数 
    * @param status        状态：-1-删除 0-预测进行中 1-预测成功  2-预测失败
    * @return
    */
   Integer countForecastByStatus(@Param(value="status") Integer status);
   
   /**
    * 分页获取进行中的预测记录
    * @param status
    * @param size          分页大小
    * @param offset        
    * @return
    */
   List<StockForecast> selectForecastByStatus(@Param(value="status") Integer status, @Param(value="size")Integer size, @Param(value="offset")Integer offset);
   
   /**
    * 更新预测记录
    * @param cast
    * @return
    */
   int updateForecast(StockForecast cast);
   
   /**
    * 统计预测总投票火钻数
    * @param forecastId    预测id
    * @return
    */
   Double selectDiamondForVoteByForecastId(@Param(value="forecastId") Integer forecastId);
   
   /**
    * 获取预测投票记录
    * @param forecastId    预测id
    * @param vtype         投票类型  0-赞成   1-反对
    * @return
    */
   List<StockForecastVote> selectVoteByForecastId(@Param(value="forecastId") Integer forecastId, @Param(value="vtype") Integer vtype);
   
   /**
    * 结算预测投票火钻数
    * @param
    * @return
    */
   int updateHarvestForVoteByForecastId(@Param(value="voteId") Integer voteId, @Param(value="vtype") Integer vtype, @Param(value="formula") String formula);

   /**
    * 更新用户投票成功率
    * @param uid
    * @return
    */
   int updateVoteFateForUser(@Param(value="uid") Integer uid);
}
