package com.huojutech.dao;

import com.huojutech.entity.Stock;
import com.huojutech.entity.StockSimple;
import org.apache.ibatis.annotations.Mapper;
import org.apache.ibatis.annotations.Param;

import java.util.List;
@Mapper
public interface StockMapper {
	/**
	 * 分页获取股票列表
	 * @param name_code		股票名称/股票代码(模糊查询使用)		
	 * @param codeList		查询股票代码列表
	 * @param fnum_orderby	预测数排序规则，0-升序  1-降序
	 * @param frate_orderby	预测成功率排序规则，0-升序  1-降序
	 * @param size			分页大小
	 * @param offset		分页的offset
	 * @return
	 */
	List<StockSimple> selectByPage(@Param(value="name_code") String name_code, @Param(value="codeList")List<String> codeList, @Param(value="fnum_orderby")Integer fnum_orderby, @Param(value="frate_orderby")Integer frate_orderby, @Param(value="size")Integer size, @Param(value="offset")Integer offset);

	/**
	 * 获取分页总记录数
	 * @param name_code		股票名称/股票代码(模糊查询使用)	
	 * @param codeList		查询股票代码列表
	 * @return
	 */
	Integer countByPage(@Param(value="name_code") String name_code, @Param(value="codeList")List<String> codeList);

	/**
	 * 获取用户自选股
	 * @param uid			用户id
	 * @return
	 */
	List<String> selectCodeForOptional(@Param(value="uid") Integer uid);
	
	/**
	 * 根据股票代码获取信息
	 * @param code          股票代码
	 * @return
	 */
	Stock selectByCode(@Param(value="code") String code);
	/**
     * 加减指定股票预测数/投票数
     * @param code          股票代码
     * @param fnum          加减预测数
     * @param vnum          加减投票数
     * @return
     */
	int updateNumberByCode(@Param(value="code") String code, @Param(value="fnum") int fnum, @Param(value="vnum") int vnum);

	/**
	 * 通过股票名称查询此条股票被投票的数量
	 * @param stockName
	 * @return
	 */
    Integer selectByName(@Param("stockName") String stockName);
	/**
	 * 热门股票,按投票数倒序
	 * @return
	 */
    List<Stock> selectHotStocks();
}
