package com.huojutech.request;

import java.io.IOException;
import java.util.UUID;

import javax.servlet.Filter;
import javax.servlet.FilterChain;
import javax.servlet.FilterConfig;
import javax.servlet.ServletException;
import javax.servlet.ServletRequest;
import javax.servlet.ServletResponse;
import javax.servlet.annotation.WebFilter;
import javax.servlet.http.Cookie;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.lang3.StringUtils;
import org.springframework.core.annotation.Order;
import org.springframework.web.util.WebUtils;

import com.alibaba.fastjson.JSONObject;
import com.huojutech.constant.Constants;
import com.huojutech.constant.SessionConstants;
import com.huojutech.utils.UrlUtils;

@Order(2)
@WebFilter(filterName = "SessionFilter", urlPatterns = "/*")
public class SessionFilter implements Filter {
	

	@Override
	public void doFilter(ServletRequest request, ServletResponse response, FilterChain chain) throws IOException, ServletException {
		// 检查sessionId的cookie是否存在,不存在即为新连接，需要初始化sessionId到cookie中
		checkRemoteSession((HttpServletRequest) request, (HttpServletResponse) response);
		chain.doFilter(request, response);
	}

	@Override
	public void destroy() {
	}

	@Override
	public void init(FilterConfig filterConfig) throws ServletException {
		
	}

	private void checkRemoteSession(HttpServletRequest request, HttpServletResponse response) {
		String sessionKey = null;
		String token = request.getParameter("token");
		
        if (StringUtils.isEmpty(token)) {
            String payload = UrlUtils.getRequestPayload(request);
            if (!StringUtils.isEmpty(payload)) {
                JSONObject jsonObject = JSONObject.parseObject(payload);
                token = jsonObject.getString("token");
            }
        }
		
		if (StringUtils.isNotEmpty(token)) {
			sessionKey = token;
		} else {
			Cookie sessionKeyCookie = WebUtils.getCookie(request, SessionConstants.SESSION_KEY_COOKIE_NAME);
			if (sessionKeyCookie == null) {
				sessionKey = UUID.randomUUID().toString();
				sessionKeyCookie = new Cookie(SessionConstants.SESSION_KEY_COOKIE_NAME, sessionKey);
			} else {
				sessionKey = sessionKeyCookie.getValue();
			}
			sessionKeyCookie.setPath("/");
			sessionKeyCookie.setDomain(Constants.CookieDomain);
			sessionKeyCookie.setMaxAge(SessionConstants.DEFAULT_SESSION_COOKIE_EXPIRE);
			sessionKeyCookie.setHttpOnly(true);
			response.addCookie(sessionKeyCookie);
		}
		RequestSessionKey.putSessionKey(sessionKey);
	}
	
}