package com.huojutech.request;

import java.lang.reflect.Method;

import javax.servlet.http.HttpServletRequest;

import org.springframework.core.MethodParameter;
import org.springframework.util.ReflectionUtils;
import org.springframework.web.bind.support.WebDataBinderFactory;
import org.springframework.web.context.request.NativeWebRequest;
import org.springframework.web.method.support.HandlerMethodArgumentResolver;
import org.springframework.web.method.support.ModelAndViewContainer;
import org.springframework.web.util.HtmlUtils;

import com.alibaba.fastjson.JSONObject;
import com.huojutech.annotation.XssParameter;
import com.huojutech.utils.UrlUtils;

public class XssHandlerMethodArgumentResolver implements HandlerMethodArgumentResolver {

	@Override
	public boolean supportsParameter(MethodParameter parameter) {
		return parameter.hasParameterAnnotation(XssParameter.class);
	}

	@SuppressWarnings({ "unchecked", "rawtypes" })
	@Override
	public Object resolveArgument(MethodParameter parameter, ModelAndViewContainer mavContainer, NativeWebRequest webRequest, WebDataBinderFactory binderFactory) throws Exception {
		String parameterName = parameter.getParameterName();
		String parameterType = parameter.getParameterType().getName();
		if ("java.lang.String".equals(parameterType)) {
			return HtmlUtils.htmlEscape(webRequest.getParameter(parameterName));
		} else {
			
			HttpServletRequest request = webRequest.getNativeRequest(HttpServletRequest.class);
			if (request.getContentType() == null || !request.getContentType().contains("application/json")) {
				throw new RuntimeException("只能application/json提交");
			}
			
			String body = UrlUtils.getRequestPayload(request);
			
			Class clazz = parameter.getParameterType();
			Method[] methods = ReflectionUtils.getAllDeclaredMethods(clazz);
			Object object = JSONObject.parseObject(body, clazz);
			
			for (Method method : methods) {
				XssUtil.htmlEscape(method, object);
			}
			
			return object;
		}
		
	}

}
