package com.huojutech.service.Impl;

import com.huojutech.dao.StockForecastCollectMapper;
import com.huojutech.dao.StockForecastMapper;
import com.huojutech.dao.StockForecastVoteMapper;
import com.huojutech.entity.StockForecast;
import com.huojutech.entity.StockForecastCollectExample;
import com.huojutech.entity.StockForecastExample;
import com.huojutech.entity.StockForecastExample.Criteria;
import com.huojutech.entity.StockForecastVoteStat;
import com.huojutech.service.StockForecastService;
import com.huojutech.utils.PageUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.util.Date;
import java.util.List;

@Service
public class StockForecastServiceImpl implements StockForecastService{
    @Autowired
    private StockForecastMapper stockForecastMapper;
    @Autowired
    private StockForecastCollectMapper stockForecastCollectMapper;
    @Autowired
    private StockForecastVoteMapper stockForecastVoteMapper;
    
    /*
     * (non-Javadoc)
     * @see com.huojutech.service.StockForecastService#selectByStockCode(java.lang.String, java.lang.Integer, java.lang.Integer, java.lang.Integer)
     */
    @Override
    public List<StockForecast> selectByStockCode(String code, Integer ltype, Integer size, Integer page) {
        StockForecastExample example = new StockForecastExample();
        Criteria c = example.createCriteria();
        c.andStatusNotEqualTo(-1);//获取未删除的预测记录
        c.andStockCodeEqualTo(code);
        if(ltype == 0){
            example.setOrderByClause("crt_time DESC");
        }else if(ltype == 1){
            example.setOrderByClause("vnum DESC");
        }
        
        PageUtil util = new PageUtil(size,page);
        example.setSize(util.getSize());
        example.setOffset(util.getOffset());
        return stockForecastMapper.selectByExampleWithBLOBs(example);
    }

    /*
     * (non-Javadoc)
     * @see com.huojutech.service.StockForecastService#countByStockCode(java.lang.String)
     */
    @Override
    public Integer countByStockCode(String code) {
        StockForecastExample example = new StockForecastExample();
        Criteria c = example.createCriteria();
        c.andStatusNotEqualTo(-1);//获取未删除的预测记录
        c.andStockCodeEqualTo(code);
        return stockForecastMapper.countByExample(example);
    }

    /*
     * (non-Javadoc)
     * @see com.huojutech.service.StockForecastService#selectById(java.lang.Integer)
     */
    @Override
    public StockForecast selectById(Integer id) {
        return stockForecastMapper.selectByPrimaryKey(id);
    }

    /*
     * (non-Javadoc)
     * @see com.huojutech.service.StockForecastService#isCollected(java.lang.Integer, java.lang.Integer)
     */
    @Override
    public boolean isCollected(Integer forecastId, Integer uid) {
        StockForecastCollectExample example = new StockForecastCollectExample();
        com.huojutech.entity.StockForecastCollectExample.Criteria c = example.createCriteria();
        c.andForecastIdEqualTo(forecastId);
        c.andUidEqualTo(uid);
        Integer total = stockForecastCollectMapper.countByExample(example);
        return total > 0 ? true:false;
    }

    /*
     * (non-Javadoc)
     * @see com.huojutech.service.StockForecastService#statByForecastId(java.lang.Integer)
     */
    @Override
    public StockForecastVoteStat statByForecastId(Integer forecastId) {
        return stockForecastVoteMapper.statByForecastId(forecastId);
    }

    /*
     * (non-Javadoc)
     * @see com.huojutech.service.StockForecastService#insertSelective(com.huojutech.entity.StockForecast)
     */
    @Override
    public int insertSelective(StockForecast cast) {
        return stockForecastMapper.insertSelective(cast);
    }
    
    /*
     * (non-Javadoc)
     * @see com.huojutech.service.StockForecastService#selectForecastCounts(java.lang.Integer)
     */
    @Override
    public Integer selectForecastCounts(Integer uid) {
        return stockForecastMapper.selectForecastCounts(uid);
    }

    /*
     * (non-Javadoc)
     * @see com.huojutech.service.StockForecastService#updateNumberById(java.lang.Integer, java.lang.Integer)
     */
    @Override
    public int updateNumberById(Integer id, Integer vnum) {
        return stockForecastMapper.updateNumberById(id, vnum);
    }

    @Override
    public List<StockForecast> findMyVoteOn(Integer uid, Integer status) {
        return stockForecastMapper.findMyVoteOn(uid,status);
    }

    @Override
    public List<StockForecast> findMyVoteOff(Integer uid, List<Integer> listStatus) {
        return stockForecastMapper.findMyVoteOff(uid,listStatus);
    }

    @Override
    public List<StockForecast> findMyForecastOn(Integer sessionId, String code, Integer status, Integer cycle, Date date) {
        StockForecastExample example = new StockForecastExample();
        Criteria criteria = example.createCriteria();
        criteria.andUidEqualTo(sessionId);
        criteria.andStockCodeEqualTo(code);
        criteria.andStatusEqualTo(status);
        criteria.andCycleEqualTo(cycle);
        criteria.andStartDateEqualTo(date);
        List<StockForecast> stockForecasts = stockForecastMapper.selectByExample(example);
        return stockForecasts;
    }

    @Override
    public Integer updateVoteHarvestById(Integer forecastId, BigDecimal harvest) {
        StockForecast stockForecast = new StockForecast();
        stockForecast.setId(forecastId);
        stockForecast.setVoteHarvest(harvest);

        return stockForecastMapper.updateByPrimaryKeySelective(stockForecast);
    }

}
