package com.huojutech.service.Impl;

import com.huojutech.dao.StockForecastTaskMapper;
import com.huojutech.entity.Stock;
import com.huojutech.entity.StockForecast;
import com.huojutech.entity.StockForecastVote;
import com.huojutech.service.StockForecastTaskService;
import com.huojutech.utils.PageUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.mongodb.core.MongoTemplate;
import org.springframework.data.mongodb.core.query.Criteria;
import org.springframework.data.mongodb.core.query.Query;
import org.springframework.stereotype.Service;

import java.util.List;

@Service
public class StockForecastTaskServiceImpl implements StockForecastTaskService{
    
    @Autowired
    private StockForecastTaskMapper stockForecastTaskMapper;
    @Autowired
    private MongoTemplate mongoTemplate;

    /*
     * (non-Javadoc)
     * @see com.huojutech.service.StockForecastTaskService#countStock()
     */
    @Override
    public Integer countStock() {
        return stockForecastTaskMapper.countStock();
    }

    /*
     * (non-Javadoc)
     * @see com.huojutech.service.StockForecastTaskService#selectStockCodes(java.lang.Integer, java.lang.Integer)
     */
    @Override
    public List<String> selectStockCodes(Integer size, Integer index) {
        PageUtil page = new PageUtil(size, index);
        return stockForecastTaskMapper.selectStockCodes(page.getSize(), page.getOffset());
    }

    /*
     * (non-Javadoc)
     * @see com.huojutech.service.StockForecastTaskService#updateFrateForStock(java.lang.String)
     */
    @Override
    public int updateFrateForStock(String code) {
        return stockForecastTaskMapper.updateFrateForStock(code);
    }

    /*
     * (non-Javadoc)
     * @see com.huojutech.service.StockForecastTaskService#countUsers()
     */
    @Override
    public Integer countUsers() {
        return stockForecastTaskMapper.countUsers();
    }

    /*
     * (non-Javadoc)
     * @see com.huojutech.service.StockForecastTaskService#selectUids(java.lang.Integer, java.lang.Integer)
     */
    @Override
    public List<Integer> selectUids(Integer size, Integer index) {
        PageUtil page = new PageUtil(size, index);
        return stockForecastTaskMapper.selectUids(page.getSize(), page.getOffset());
    }

    /*
     * (non-Javadoc)
     * @see com.huojutech.service.StockForecastTaskService#updateFrateForUser(java.lang.Integer)
     */
    @Override
    public int updateFrateForUser(Integer uid) {
        return stockForecastTaskMapper.updateFrateForUser(uid);
    }

    @Override
    public int updateVoteFateForUser(Integer uid) {
        return stockForecastTaskMapper.updateVoteFateForUser(uid);
    }

    /*
     * (non-Javadoc)
     * @see com.huojutech.service.StockForecastTaskService#selectConfigByKey(java.lang.String)
     */
    @Override
    public String selectConfigByKey(String key) {
        return stockForecastTaskMapper.selectConfigByKey(key);
    }

    /*
     * (non-Javadoc)
     * @see com.huojutech.service.StockForecastTaskService#countForecastByStatus(java.lang.Integer)
     */
    @Override
    public Integer countForecastByStatus(Integer status) {
        return stockForecastTaskMapper.countForecastByStatus(status);
    }

    /*
     * (non-Javadoc)
     * @see com.huojutech.service.StockForecastTaskService#selectForecastByStatus(java.lang.Integer, java.lang.Integer, java.lang.Integer)
     */
    @Override
    public List<StockForecast> selectForecastByStatus(Integer status, Integer size, Integer index) {
        PageUtil page = new PageUtil(size, index);
        return stockForecastTaskMapper.selectForecastByStatus(status, page.getSize(), page.getOffset());
    }

    /*
     * (non-Javadoc)
     * @see com.huojutech.service.StockForecastTaskService#updateForecast(com.huojutech.entity.StockForecast)
     */
    @Override
    public int updateForecast(StockForecast cast) {
        return stockForecastTaskMapper.updateForecast(cast);
    }

    /*
     * (non-Javadoc)
     * @see com.huojutech.service.StockForecastTaskService#selectStockByCodeFromMongoDB(java.lang.String)
     */
    @Override
    public Stock selectStockByCodeFromMongoDB(String code) {
        Query query = new Query(Criteria.where("code").is(code));
        /* 从MongoDB获取股票实时数据 */
        Stock s = mongoTemplate.findOne(query, Stock.class, "stock_today_all");
        return s;
    }

    /*
     * (non-Javadoc)
     * @see com.huojutech.service.StockForecastTaskService#selectDiamondForVoteByForecastId(java.lang.Integer)
     */
    @Override
    public Double selectDiamondForVoteByForecastId(Integer forecastId) {
        return stockForecastTaskMapper.selectDiamondForVoteByForecastId(forecastId);
    }

    /*
     * (non-Javadoc)
     * @see com.huojutech.service.StockForecastTaskService#selectVoteByForecastId(java.lang.Integer, java.lang.Integer)
     */
    @Override
    public List<StockForecastVote> selectVoteByForecastId(Integer forecastId, Integer vtype) {
        return stockForecastTaskMapper.selectVoteByForecastId(forecastId, vtype);
    }

    /*
     * (non-Javadoc)
     * @see com.huojutech.service.StockForecastTaskService#updateHarvestForVoteByForecastId(java.lang.Integer, java.lang.Integer, java.lang.String)
     */
    @Override
    public int updateHarvestForVoteByForecastId(Integer voteId, Integer vtype, String formula) {
        return stockForecastTaskMapper.updateHarvestForVoteByForecastId(voteId, vtype, formula);
    }

}
