package com.huojutech.service.Impl;

import com.github.pagehelper.StringUtil;
import com.huojutech.constant.MongoConstants;
import com.huojutech.dao.StockMapper;
import com.huojutech.dto.StockInfoADTO;
import com.huojutech.dto.StockInfoDTO;
import com.huojutech.entity.Stock;
import com.huojutech.entity.StockSimple;
import com.huojutech.service.StockService;
import com.huojutech.utils.PageUtil;
import com.huojutech.vo.StockKVO;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Sort;
import org.springframework.data.domain.Sort.Direction;
import org.springframework.data.mongodb.core.MongoTemplate;
import org.springframework.data.mongodb.core.query.Criteria;
import org.springframework.data.mongodb.core.query.Query;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.List;

/**
 * 股票service实现层
 * @author zhoucong
 *
 */
@Service
public class StockServiceImpl implements StockService{
	private static Logger logger = LoggerFactory.getLogger(StockServiceImpl.class);


	@Autowired
	private StockMapper stockMapper;
	@Autowired
	private MongoTemplate mongoTemplate;

	/*
	 * (non-Javadoc)
	 * @see com.huojutech.service.StockService#selectByPage(java.lang.String, java.util.List, java.lang.Integer, java.lang.Integer, java.lang.Integer, java.lang.Integer, java.lang.Integer, java.lang.Integer)
	 */
	//TODO 当参数过多时需要考虑封装一个查询类
	@Override
	public List<StockSimple> selectByPage(String name_code, List<String> codeList, Integer tradeOrderby, Integer cpOrderby, Integer fnumOrderby, Integer frateOrderby, Integer size, Integer page) {
		PageUtil util = new PageUtil(size, page);
		Sort sort = null;
		if (tradeOrderby != null) {
			sort = new Sort(tradeOrderby == 1 ? Direction.DESC:Direction.ASC, "trade");
			if (cpOrderby != null) {
				sort.and(new Sort(tradeOrderby == 1 ? Direction.DESC:Direction.ASC, "changepercent"));
			}
		}else if (cpOrderby != null) {
			sort = new Sort(cpOrderby == 1 ? Direction.DESC:Direction.ASC, "changepercent");
		}
		List<StockSimple> datalist = null;
		if (sort != null) {
			/* 当按现价排序时从mondb中获取股票数据 */
			datalist =  getListFromMongoDB(name_code, codeList, sort, util.getSize(), util.getOffset());
			if (!CollectionUtils.isEmpty(datalist)) {
				codeList = new ArrayList<String>();
				for (StockSimple stockSimple : datalist) {
					codeList.add(stockSimple.getCode());
				}
				/* 从mysql总获取对应股票的预测数和预测成功率 */
				List<StockSimple> stocks = stockMapper.selectByPage(null, codeList, null, null, null, null);
				for (StockSimple data : datalist) {
					for (StockSimple stock : stocks) {
						if (data.getCode().equals(stock.getCode())) {
							data.setFnum(stock.getFnum());
							data.setFrateSuccess(stock.getFrateSuccess());  //TODO 出现四层以上结构是 最好能优化
							data.setTrade(data.getNow());
						}
					}
				}
			}
		}else{
			/* 从mysql中获取股票数据 */
			datalist = stockMapper.selectByPage(name_code, codeList, fnumOrderby, frateOrderby, util.getSize(), util.getOffset());
			if (!CollectionUtils.isEmpty(datalist)) {
				codeList = new ArrayList<String>();
				for (StockSimple stockSimple : datalist) {
					codeList.add(stockSimple.getCode());
				}
				/* 从mongoDB中获取对应股票的现价和涨幅 */
				List<StockSimple> stocks = getListFromMongoDB(null, codeList, null, null, null);
				for (StockSimple data : datalist) {
					for (StockSimple stock : stocks) {
						if (data.getCode().equals(stock.getCode())) {
							Double now = stock.getNow();
							Double open = stock.getOpen();
							Double close = stock.getClose();
							double v = (now - close)/close;
							data.setTrade(stock.getNow());
							data.setChangepercent(stock.getChangepercent());
							data.setOpen(open);
						}
					}
				}
			}
		}
		
		return datalist;
	}

	/*
	 * (non-Javadoc)
	 * @see com.huojutech.service.StockService#countByPage(java.lang.String, java.util.List)
	 */
	@Override
	public Integer countByPage(String name_code, List<String> codeList) {
		return stockMapper.countByPage(name_code, codeList);
	}

	/*
	 * (non-Javadoc)
	 * @see com.huojutech.service.StockService#selectCodeForOptional(java.lang.Integer)
	 */
	@Override
	public List<String> selectCodeForOptional(Integer uid) {
		return stockMapper.selectCodeForOptional(uid);
	}
	
	/*
	 * (non-Javadoc)
	 * @see com.huojutech.service.StockService#selectByCode(java.lang.String)
	 */
	@Override
    public Stock selectByCode(String code) {
	    Stock stock = stockMapper.selectByCode(code);
	    if (stock != null) {
	        Query query = new Query(Criteria.where("code").is(stock.getCode()));
	        /* 从MongoDB获取股票实时数据 */
			StockInfoDTO s1 = mongoTemplate.findOne(query, StockInfoDTO.class, "stock_today_all");
	        StockInfoADTO s = mongoTemplate.findOne(query, StockInfoADTO.class, "stock_basic");
	        if(s != null){
				Double nowPrice = s1.getNow();//当前价
				Double open = s1.getOpen();//开盘价
				Double close = s1.getClose();//前一天的收盘价
				double diff = nowPrice - close;
				BigDecimal b = new BigDecimal(diff);
				diff   =   b.setScale(2,BigDecimal.ROUND_HALF_UP).doubleValue();

				Double pe = s.getPe();//市赢率

				Double totals = s.getTotals();//当前市值=totals* 当前股价
				Double outstanding = s.getOutstanding();//流通市值=outStanding*当前股价
				double nmc=0.0;
				double mktcap=0.0;
				if(nowPrice.equals(0.0)){
					 nmc = totals * 1;
					 mktcap = outstanding * 1;
				}else{
					mktcap = totals * nowPrice;
					nmc = outstanding * nowPrice;
				}

				stock.setDiff(diff);
				stock.setChangepercent(s1.getChangepercent());
				stock.setTrade(nowPrice);
				stock.setPer(pe);
				stock.setNmc(nmc);
				stock.setMktcap(mktcap);
				stock.setHigh(s1.getHigh());
				stock.setLow(s1.getLow());
				stock.setOpen(s1.getOpen());
//	            s.setFnum(stock.getFnum());
//	            s.setVnum(stock.getVnum());
//	            s.setFrateSuccess(stock.getFrateSuccess());
//	            stock = s;
	        }
        }
        return stock;
    }
	
	/*
	 * (non-Javadoc)
	 * @see com.huojutech.service.StockService#updateNumberByCode(java.lang.String, java.lang.Integer, java.lang.Integer)
	 */
	@Override
    public int updateNumberByCode(String code, int fnum, int vnum) {
        return stockMapper.updateNumberByCode(code, fnum, vnum);
    }
	
	/**
	 * 从MongoDB中获取列表数据
	 * @param name_code				股票名称
	 * @param codeList			指定获取股票的代码列表
	 * @param sort
	 * @param size				分页大小
	 * @param offset			相当于从哪条记录开始
	 */
	private List<StockSimple> getListFromMongoDB(String name_code, List<String> codeList, Sort sort, Integer size, Integer offset){
		Query query = new Query();
		if (!StringUtil.isEmpty(name_code)) {
			query.addCriteria(Criteria.where("name").regex(name_code));
			query.addCriteria(Criteria.where("code").regex(name_code));
		}
		if (!CollectionUtils.isEmpty(codeList)) {
			query.addCriteria(Criteria.where("code").in(codeList));
		}
		if (sort != null) {
			query.with(sort);
		}
		if (offset != null && offset > 0) {
			query.skip(offset);
		}
		if (size != null && size > 0) {
			query.limit(size);
		}
		return mongoTemplate.find(query, StockSimple.class, MongoConstants.TABLE_STOCK_TODAY);
	}
	
    public List<StockKVO> getStockKList(String type, String code, String startDate, String endDate) {
        Query query = new Query();
        if (!StringUtil.isEmpty(code)) {
            query.addCriteria(Criteria.where("scode").is(code));
        }
        if (!StringUtils.isEmpty(startDate) && !StringUtils.isEmpty(endDate)) {
            query.addCriteria(Criteria.where("date").gte(startDate).lte(endDate));
        }
        
        query.with(new Sort(Direction.DESC, "date"));
        
        String table = MongoConstants.TABLE_STOCK_EXT_DAY_K;
        if ("d".equals(type)) {
            table = MongoConstants.TABLE_STOCK_EXT_DAY_K;
        } else if ("w".equals(type)) {
            table = MongoConstants.TABLE_STOCK_EXT_WEEK_K;
        } else {
            table = MongoConstants.TABLE_STOCK_EXT_MONTH_K;
        }
        return mongoTemplate.find(query, StockKVO.class, table);
    }

	@Override
	public List<Stock> selectHotStocks() {
		return stockMapper.selectHotStocks();
	}
}
