package com.huojutech.service;

import com.huojutech.entity.StockForecast;
import com.huojutech.entity.StockForecastVoteStat;

import java.math.BigDecimal;
import java.util.Date;
import java.util.List;

/**
 * 股票预测服务
 * @author zhoucong
 *
 */
public interface StockForecastService {
    /**
     * 获取单股预测列表
     * @param code          股票代码
     * @param ltype         列表类型   0-最新；1-最热  2-最准
     * @param size          分页大小
     * @param page          页码
     * @return
     */
    List<StockForecast> selectByStockCode(String code, Integer ltype, Integer size, Integer page);
    /**
     * 单股预测列表总记录数
     * @param code          股票代码
     * @return
     */
    Integer countByStockCode(String code);
    /**
     * 根据id获取股票预测详情
     * @param id
     * @return
     */
    StockForecast selectById(Integer id);
    /**
     * 判断用户是否已经收藏指定股票预测
     * @param forecastId    股票预测id
     * @param uid           用户id
     * @return
     */
    boolean isCollected(Integer forecastId, Integer uid);
    
    /**
     * 统计预测投票情况
     * @param forecastId
     * @return
     */
    StockForecastVoteStat statByForecastId(Integer forecastId);
    /**
     * 添加股票预测
     * @param cast
     * @return
     */
    int insertSelective(StockForecast cast);
    /**
     * 统计预测数量
     * @param uid
     * @return
     */
    Integer selectForecastCounts(Integer uid);
    
    /**
     * 加减预测投票数
     * @param id
     * @param vnum
     * @return
     */
    int updateNumberById(Integer id, Integer vnum);

    /**
     * 查询正在投票中的股票
     * @param uid
     * @param status
     * @return
     */
    List<StockForecast> findMyVoteOn(Integer uid, Integer status);

    /**
     * 查找已经结束投票的股票
     * @param uid
     * @param listStatus
     * @return
     */
    List<StockForecast> findMyVoteOff(Integer uid, List<Integer> listStatus);

    /**
     * 查询正在预测中的股票
     * @param sessionId
     * @param code
     *@param status 预测状态  -1-删除 0-预测进行中  @return
     */
    List<StockForecast> findMyForecastOn(Integer sessionId, String code, Integer status, Integer cycle, Date date);

    /**
     * 通过预测id更新投票收益
     * @param forecastId
     * @param harvest
     * @return
     */
    Integer updateVoteHarvestById(Integer forecastId, BigDecimal harvest);
}
