package com.huojutech.service;

import com.huojutech.entity.Stock;
import com.huojutech.entity.StockForecast;
import com.huojutech.entity.StockForecastVote;

import java.util.List;

public interface StockForecastTaskService {
    /**
     * 统计股票总记录数 
     * @return
     */
    Integer countStock();
    
    /**
     * 获取股票代码
     * @param size          分页大小
     * @param index         页码，第一页为1         
     * @return
     */
    List<String> selectStockCodes(Integer size, Integer index);
    
    /**
     * 更新股票预测成功率
     * @param code          股票代码
     * @return
     */
    int updateFrateForStock(String code);



    /**
     * 获取用户总记录数
     * @return
     */
    Integer countUsers();
    
    /**
     * 获取用户id
     * @param size          分页大小
     * @param index         页码，第一页为1
     * @return
     */
    List<Integer> selectUids(Integer size, Integer index);
    
    /**
     * 更新用户预测成功率
     * @param uid           用户id
     * @return
     */
    int updateFrateForUser(Integer uid);

    /**
     * 更新用户投票成功率
     * @param uid
     * @return
     */
    int updateVoteFateForUser(Integer uid);

    /**
     * 获取配置信息
     * @param key
     * @return
     */
    String selectConfigByKey(String key);
    
    /**
     * 统计进行中的预测记录数 
     * @param status        状态：-1-删除 0-预测进行中 1-预测成功  2-预测失败
     * @return
     */
    Integer countForecastByStatus(Integer status);
    
    /**
     * 分页获取进行中的预测记录
     * @param status
     * @param size          分页大小
     * @param index         页码，第一页为1
     * @return
     */
    List<StockForecast> selectForecastByStatus(Integer status, Integer size, Integer index);
    
    /**
     * 更新预测记录
     * @param cast
     * @return
     */
    int updateForecast(StockForecast cast);
    
    /**
     * 从mongoDB获取股票信息
     * @param code
     * @return
     */
    Stock selectStockByCodeFromMongoDB(String code);
    
    /**
     * 统计预测总投票火钻数
     * @param forecastId    预测id
     * @return
     */
    Double selectDiamondForVoteByForecastId(Integer forecastId);
    
    /**
     * 获取预测投票记录
     * @param forecastId    预测id
     * @param vtype         投票类型  0-赞成   1-反对
     * @return
     */
    List<StockForecastVote> selectVoteByForecastId(Integer forecastId, Integer vtype);
    
    /**
     * 结算预测投票火钻数
     * @param forecastId    预测id
     * @param vtype         投票类型  0-赞成   1-反对
     * @param formula       结算公式
     * @return
     */
    int updateHarvestForVoteByForecastId(Integer voteId, Integer vtype, String formula);


}
