package com.huojutech.service;

import com.huojutech.entity.Stock;
import com.huojutech.entity.StockSimple;
import com.huojutech.vo.StockKVO;

import java.util.List;

public interface StockService {
	/**
	 * 分页获取股票列表
	 * @param name_code		股票名称/股票代码(模糊查询使用)	
	 * @param codeList		查询股票代码列表
	 * @param priceOrderby	股价排序规则，0-升序  1-降序 (注：无法与fnumOrderby、frateOrderby合用)
	 * @param cpOrderby		涨幅排序规则，0-升序  1-降序 (注：无法与fnumOrderby、frateOrderby合用)
	 * @param fnumOrderby	预测数排序规则，0-升序  1-降序 (注：无法与priceOrderby、cpOrderby合用)
	 * @param frateOrderby	预测成功率排序规则，0-升序  1-降序 (注：无法与priceOrderby、cpOrderby合用)
	 * @param size			分页大小
	 * @param page			页码
	 * @return
	 */
	List<StockSimple> selectByPage(String name_code, List<String> codeList, Integer priceOrderby, Integer cpOrderby, Integer fnumOrderby, Integer frateOrderby, Integer size, Integer page);
	
	/**
	 * 获取分页总记录数
	 * @param name_code		股票名称/股票代码(模糊查询使用)
	 * @param codeList		查询股票代码列表
	 * @return
	 */
	Integer countByPage(String name_code, List<String> codeList);
	
	/**
	 * 分页获取自选股列表
	 * @param uid			用户id
	 * @return
	 */
	List<String> selectCodeForOptional(Integer uid);
	
	/**
     * 根据股票代码获取信息
     * @param code          股票代码
     * @return
     */
    Stock selectByCode(String code);
    
    /**
     * 加减指定股票预测数/投票数
     * @param code          股票代码
     * @param fnum          加减预测数
     * @param vnum          加减投票数
     * @return
     */
    int updateNumberByCode(String code, int fnum, int vnum);
    
    
    List<StockKVO> getStockKList(String type, String code, String startDate, String endDate);

	/**
	 * 热门股票,按投票数倒序6个
	 * @return
	 */
    List<Stock> selectHotStocks();
}
