package com.huojutech.task;

import com.huojutech.constant.Constants;
import com.huojutech.entity.Stock;
import com.huojutech.entity.StockForecast;
import com.huojutech.entity.StockForecastVote;
import com.huojutech.entity.User;
import com.huojutech.service.StockForecastTaskService;
import com.huojutech.service.StockForecastVoteService;
import com.huojutech.service.UserService;
import org.apache.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Configuration;
import org.springframework.scheduling.annotation.EnableScheduling;
import org.springframework.scheduling.annotation.Scheduled;

import javax.script.ScriptEngine;
import javax.script.ScriptEngineManager;
import java.math.BigDecimal;
import java.util.Date;
import java.util.List;

@Configuration
@EnableScheduling
public class StockForecastTask {
    private final Logger logger = Logger.getLogger(getClass());
    
    @Autowired
    private StockForecastTaskService stockForecastTaskService;

    @Autowired
    private UserService userService;

    @Autowired
    private StockForecastVoteService stockForecastVoteService;
    
    private static ScriptEngine jse = new ScriptEngineManager().getEngineByName("JavaScript");
    
    //TODO 判断预测是否成功任务-每天凌晨0点
    @Scheduled(cron = "0 0 0 * * ?")
    public void isSuccessForForecastTask() {
        logger.info("判断预测是否成功任务开始>>>>>");
        //每次获取记录数及初始页码
        Integer size = 1000, index = 1;
        //起始记录位置
        Integer offset = 0;
        try {
            //获取预测火钻生成公式
            String formula = stockForecastTaskService.selectConfigByKey(Constants.CONFIG_FORECAST_FORMULA);
            //获取预测投票火钻生成公式
            String vformula = stockForecastTaskService.selectConfigByKey(Constants.CONFIG_FORECAST_VOTE_FORMULA);
            //获取预测浮动范围
            Double f = Double.valueOf(stockForecastTaskService.selectConfigByKey(Constants.CONFIG_FORECAST_FLOAT));
            
            double changepercent = 0;
            Double vbet = null;
            Stock stock = null;
            //获取待计算预测列表总记录数
            Integer total = stockForecastTaskService.countForecastByStatus(0);
            while(total > offset){
                List<StockForecast> list = stockForecastTaskService.selectForecastByStatus(0, size, index++);
                for (StockForecast cast : list) {
                    //获取预测股票当天的股价信息 #bet#*(1+#changepercent#)+#vbet#*0.02 TODO 计算方法有问题！！！！
                    stock = stockForecastTaskService.selectStockByCodeFromMongoDB(cast.getStockCode());
                    //计算价位变化幅度
                    changepercent = Math.abs(cast.getCurrentPrice().subtract(cast.getTargetPrice()).divide(cast.getCurrentPrice(), 6, BigDecimal.ROUND_HALF_UP).doubleValue());
                    
                    if((cast.getExpect() == 0 && stock.getHigh() >= cast.getTargetPrice().doubleValue() / (1+f))
                            || (cast.getExpect() == 1 && stock.getLow() <= cast.getTargetPrice().doubleValue() * (1+f))){//看多
                        //获取总投票火钻数
                        vbet = stockForecastTaskService.selectDiamondForVoteByForecastId(cast.getId());
                        if (vbet==null){
                            vbet=0.00;
                        }
                        formula = formula.replace("#bet#", cast.getBet()+"").replace("#changepercent#", String.valueOf(changepercent)).replace("#vbet#", vbet+"");
                        BigDecimal harvest = new BigDecimal(Double.valueOf(jse.eval(formula) + ""));
                        cast.setHarvest(harvest);
                        cast.setStatus(1);//标记为预测成功
                        //预测成功用户加火钻
                        BigDecimal bet = cast.getBet();
                        BigDecimal add = bet.add(harvest);
                        User user1 = userService.selectUserById(cast.getUid());
                        BigDecimal amount = user1.getAmount();
                        BigDecimal add1 = amount.add(add);
                        User user = new User();
                        user.setId(cast.getUid());
                        user.setAmount(add1);
                        userService.updateUser(user);
                    }else if(cast.getEndDate() != null && new Date().after(cast.getEndDate())){//判断预测周期是否结束
                        cast.setStatus(2);//标记为预测失败
                    }
                    if (cast.getStatus()!=null){
                        if (cast.getStatus() == 1 || cast.getStatus() == 2) {
                            stockForecastTaskService.updateForecast(cast);
                            //预测成功结算投票记录
                            List<StockForecastVote> stockForecastVoteList = stockForecastVoteService.selectByForecastId(cast.getId());
                            for(int i=0;i<stockForecastVoteList.size();i++){
                                StockForecastVote stockForecastVote = stockForecastVoteList.get(i);
                                Integer voteId = stockForecastVote.getId();
                                BigDecimal bet = stockForecastVote.getBet();
                                String replace = vformula.replace("#bet#", bet + "").replace("#changepercent#", String.valueOf(changepercent));
                                Integer m = cast.getStatus() == 1 ? 0 : 1;
                                //相等则投票成功
                                if (m.equals(stockForecastVote.getVtype())){
                                    stockForecastTaskService.updateHarvestForVoteByForecastId(voteId,m,replace);
                                    //更新用户表的火钻数
                                    Integer uid = stockForecastVote.getUid();
                                    User user = userService.selectUserById(uid);
                                    BigDecimal havest = new BigDecimal(String.format("%.6f", jse.eval(replace)));
                                    BigDecimal amount = user.getAmount();
                                    BigDecimal add = amount.add(havest);
                                    BigDecimal add1 = add.add(bet);
                                    user.setAmount(add1);
                                    userService.updateUser(user);
                                    //更新投票表的状态，为投票成功
                                    stockForecastVote.setStatus(1);
                                }else{
                                    //更新投票表的状态，为投票失败
                                    stockForecastVote.setStatus(2);
                                }
                                stockForecastVoteService.updateVote(stockForecastVote);
                            }
                        }
                    }
                }
                offset = size * (index - 1);
            }
            
        } catch (Exception e) {
            logger.error("判断预测是否成功任务错误："+e.getMessage(), e);
        }
        logger.info("<<<<<判断预测是否成功任务结束");
    }
    
    // 股票 的每预测成功率计算任务-天凌晨2点
    @Scheduled(cron = "0 0 2 * * ?")
    public void successRateForStockCalculateTask() {
        logger.info("股票 的预测成功率计算任务开始>>>>>");
        //每次获取记录数及初始页码
        Integer size = 1000, index = 1;
        //起始记录位置
        Integer offset = 0;
        try {
            /* 获取股票总记录数 */
            Integer total = stockForecastTaskService.countStock();
            while(total > offset){
                List<String> codes = stockForecastTaskService.selectStockCodes(size, index++);
                for (String code : codes) {
                    stockForecastTaskService.updateFrateForStock(code);
                }
                offset = size * (index - 1);
            }
        } catch (Exception e) {
            logger.error("股票 的预测成功率计算任务错误："+e.getMessage(), e);
        }
        logger.info("<<<<<股票 的预测成功率计算任务结束");
    }
    
    // 用户 的预测成功率计算任务-每天凌晨2点
    @Scheduled(cron = "0 0 2 * * ?")
    public void successRateForUserCalculateTask() {
        logger.info("用户 的预测成功率计算任务开始>>>>>");
        //每次获取记录数及初始页码
        Integer size = 1000, index = 1;
        //起始记录位置
        Integer offset = 0;
        
        try {
            /* 获取用户总记录数 */
            Integer total = stockForecastTaskService.countUsers();
            while(total > offset){
                List<Integer> uids = stockForecastTaskService.selectUids(size, index++);
                for (Integer uid : uids) {
                    stockForecastTaskService.updateFrateForUser(uid);
                    stockForecastTaskService.updateVoteFateForUser(uid);
                }

                offset = size * (index - 1);
            }
        } catch (Exception e) {
            logger.error("用户 的预测成功率计算任务开始："+e.getMessage(), e);
        }
        
        logger.info("<<<<<用户 的预测成功率计算任务结束");
    }
}
