package com.huojutech.utils;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;
import java.util.GregorianCalendar;
import java.util.Locale;
/**
 * 日期工具类
 * @author zhoucong
 *
 */
public class DateUtil {
    private static Logger logger = LoggerFactory.getLogger(DateUtil.class);
    
    private static SimpleDateFormat shortDateFormat = new SimpleDateFormat("yyyy-MM-dd");
    /**
     * 日期转换str->date
     * @param dateStr
     *            default:yyyy-MM-dd
     * @param pattern
     * @return
     */
    public static Date str2Date(String dateStr, String pattern) {
        Date date = null;
        SimpleDateFormat format = null;
        if (pattern == null || ("").equals(pattern)) {
            format = shortDateFormat;
        } else {
            format = new SimpleDateFormat(pattern, Locale.US);
        }
        try {
            synchronized (format) {
                // SimpleDateFormat is not thread safe
                date = format.parse(dateStr);
            }
        } catch (ParseException e) {
            logger.error(e.getMessage(),e);
        }
        return date;
    }
    
    /**
     * 日期转换
     * date->str
     * @param date
     * @param pattern
     * @return
     */
    public static String date2Str(Date date, String pattern) {
        if(date != null){
            SimpleDateFormat dateFormat = new SimpleDateFormat(pattern);
            return dateFormat.format(date);
        } else {
            return "";
        }
    }
    
    /**
     * 得到类似微博的时间格式
     * @param date
     * @return
     */
    public static String getWeiboTime2(Date date) {
        String shortstring = null;

        if (date == null)
            return shortstring;

        long now = Calendar.getInstance().getTimeInMillis();
        long deltime = (now - date.getTime()) / 1000;
        if (deltime > 2 * 24 * 60 * 60) {
            shortstring = date2Str(date, "MM月dd日 HH:mm");
        } else if (deltime > 24 * 60 * 60) {
            shortstring = "昨天 " + date2Str(date, "HH:mm");
        } else if (deltime > 60 * 60) {
            shortstring = (int) (deltime / (60 * 60)) + "小时前";
        } else if (deltime > 60) {
            shortstring = (int) (deltime / (60)) + "分钟前";
        } else if (deltime > 1) {
            shortstring = deltime + "秒前";
        } else {
            shortstring = "刚刚";
        }
        return shortstring;
    }
    
    /**
     * 得到类似微博的时间格式 比较早的时间不加几点几分
     * @param date
     * @return
     */
    public static String getWeiboTime(Date date) {
        System.out.println(date2Str(date, "yyyy-MM-dd HH:mm:ss"));
        String shortstring = null;

        if (date == null)
            return shortstring;

        long now = Calendar.getInstance().getTimeInMillis();
        long deltime = (now - date.getTime()) / 1000;
        if (deltime > 2 * 24 * 60 * 60) {
            shortstring = date2Str(date, "");
        } else if (deltime > 24 * 60 * 60) {
            shortstring = "昨天 ";// + date2Str(date, "HH:mm");
        } else if (deltime > 60 * 60) {
            shortstring = (int) (deltime / (60 * 60)) + "小时前";
        } else if (deltime > 60) {
            shortstring = (int) (deltime / (60)) + "分钟前";
        } else if (deltime > 1) {
            shortstring = deltime + "秒前";
        } else {
            shortstring = "刚刚";
        }
        return shortstring;
    }
    
    /**
     * 获取当前日期的第一天
     * @param date 当前日期
     * @return 当前日期的第一天
     */
    public static Date getFirstDayByDate(Date date){
        if(date == null){
            return null;
        }
        Calendar cal = Calendar.getInstance();
        cal.setTime(date);
        cal.set(Calendar.DAY_OF_MONTH, 1);
        return cal.getTime();
    }
    
    /**
     * 日期提前或者推迟N天
     * @param date 日期
     * @param amount 计量数
     * @return 处理后的日期
     */
    public static Date addDate(Date date, int amount){
        
        if(date == null){
            return null;
        }
        
        Calendar cal = Calendar.getInstance();
        cal.setTime(date);
        cal.add(Calendar.DAY_OF_MONTH, amount);
        return cal.getTime();
    }

    /**
     * 求2个日期之间的差值 a-b
     * @param aDate
     * @param bDate
     * @return
     */
    public static long subDates(Date aDate, Date bDate){
        long sub = 0;
        
        if (aDate != null && bDate != null) {
            sub = aDate.getTime() - bDate.getTime();
        }
        
        return sub;
    }
    
    /**
     * 闰年判断
     * @param year
     * @return
     */
    public static boolean isLeapYear(int year) {

        GregorianCalendar calendar = new GregorianCalendar();

        return calendar.isLeapYear(year);

    }
    
    /**
     * 判断是否为日期
     * @param date
     * @param pattern
     * @return
     */
    public static boolean isDate(String date, String pattern){
        try {
            SimpleDateFormat format=new SimpleDateFormat(pattern);
            format.parse(date);
        } catch (Exception e) {
            return false;
        }
        return true;
    }
    
    /**
     * 获取两个时间之间间隔的天数
     * @return
     */
    public static Integer getDays(Date date1, Date date2){
        long sub = Math.abs(subDates(date1, date2));
        
        long days = sub/(24*60*60*1000);
        if (sub%(24*60*60*1000) > 0) {
            days++;
        }
        
        return Integer.valueOf(days+"");
    }

    public static int diffDate(Date startDate, Date endDate) {
        long endMillis = endDate.getTime();
        long startMillis = startDate.getTime();
        long s = (endMillis - startMillis) / (24 * 3600 * 1000);
        return (int) s;
    }
}
